@@ -2,7 +2,7 @@
 /* GdkPixbuf library - SVG image loader
  *
  * Copyright (C) 2002 Matthias Clasen
- * Copyright (C) 2002 Dom Lachowicz
+ * Copyright (C) 2002-2004 Dom Lachowicz
  *
  * Authors: Matthias Clasen <maclas@gmx.de>
  *          Dom Lachowicz <cinamod@hotmail.com>
@@ -30,7 +30,6 @@
 
 typedef struct {
         RsvgHandle                 *handle;
-        GdkPixbuf                  *pixbuf;
 
         GdkPixbufModuleUpdatedFunc  updated_func;
         GdkPixbufModulePreparedFunc prepared_func;
@@ -62,7 +61,7 @@ rsvg_propegate_error (GError ** err,
 
 static gpointer
 gdk_pixbuf__svg_image_begin_load (GdkPixbufModuleSizeFunc size_func,
-                                  GdkPixbufModulePreparedFunc prepared_func, 
+                                  GdkPixbufModulePreparedFunc prepared_func,
                                   GdkPixbufModuleUpdatedFunc  updated_func,
                                   gpointer user_data,
                                   GError **error)
@@ -82,6 +81,24 @@ gdk_pixbuf__svg_image_begin_load (GdkPixbufModuleSizeFunc size_func,
         return context;
 }
 
+static void
+emit_updated (SvgContext *context, GdkPixbuf *pixbuf)
+{
+        if (context->updated_func != NULL)
+                (* context->updated_func) (pixbuf,
+                                           0, 0,
+                                           gdk_pixbuf_get_width (pixbuf),
+                                           gdk_pixbuf_get_height (pixbuf),
+                                           context->user_data);
+}
+
+static void
+emit_prepared (SvgContext *context, GdkPixbuf *pixbuf)
+{
+        if (context->prepared_func != NULL)
+                (* context->prepared_func) (pixbuf, NULL, context->user_data);
+}
+
 static gboolean
 gdk_pixbuf__svg_image_load_increment (gpointer data,
 				      const guchar *buf, guint size,
@@ -101,57 +118,57 @@ gdk_pixbuf__svg_image_load_increment (gpointer data,
                 else
                         context->handle = rsvg_handle_new ();
 
-                if (!context->handle)
+                if (!context->handle) {
+                        rsvg_propegate_error (error, _("Error displaying image"), ERROR_DISPLAYING_IMAGE);
                         return FALSE;
+                }
 
                 rsvg_handle_set_size_callback (context->handle, context->size_func, context->user_data, NULL);
         }
 
+        if (!context->handle) {
+                rsvg_propegate_error (error, _("Error displaying image"), ERROR_DISPLAYING_IMAGE);
+                return FALSE;
+        }
+
         if (!rsvg_handle_write (context->handle, buf, size, error)) {
                 rsvg_propegate_error (error, _("Error writing"), ERROR_WRITING);
                 return FALSE;
         }
 
-        context->pixbuf = rsvg_handle_get_pixbuf (context->handle);
-  
-        if (context->pixbuf != NULL && context->prepared_func != NULL)
-                (* context->prepared_func) (context->pixbuf, NULL, context->user_data);        
-  
         return TRUE;
 }
 
 static gboolean
 gdk_pixbuf__svg_image_stop_load (gpointer data, GError **error)
 {
-        SvgContext *context = (SvgContext *)data;  
+        SvgContext *context = (SvgContext *)data;
+        GdkPixbuf *pixbuf;
         gboolean result = TRUE;
 
         if (error)
                 *error = NULL;
 
-        rsvg_handle_close (context->handle, error);
-
-        if (context->pixbuf == NULL) {
-                context->pixbuf = rsvg_handle_get_pixbuf (context->handle);
-    
-                if (context->pixbuf != NULL && context->prepared_func != NULL)
-                        (* context->prepared_func) (context->pixbuf, NULL, context->user_data);
+        if (!context->handle) {
+                rsvg_propegate_error (error, _("Error displaying image"), ERROR_DISPLAYING_IMAGE);
+                return FALSE;
         }
 
-        if (context->pixbuf != NULL && context->updated_func != NULL)
-                (* context->updated_func) (context->pixbuf, 
-                                           0, 0, 
-                                           gdk_pixbuf_get_width (context->pixbuf), 
-                                           gdk_pixbuf_get_height (context->pixbuf), 
-                                           context->user_data);
-        else if (!context->pixbuf) {
+        rsvg_handle_close (context->handle, error);
+
+        pixbuf = rsvg_handle_get_pixbuf (context->handle);
+
+        if (pixbuf != NULL) {
+                emit_prepared (context, pixbuf);
+                emit_updated (context, pixbuf);
+                g_object_unref (pixbuf);
+        }
+        else {
                 rsvg_propegate_error (error, _("Error displaying image"), ERROR_DISPLAYING_IMAGE);
                 result = FALSE;
         }
 
         rsvg_handle_free (context->handle);
-        if (context->pixbuf)
-                g_object_unref (context->pixbuf);
         g_free (context);
 
         return TRUE;
@@ -160,11 +177,25 @@ gdk_pixbuf__svg_image_stop_load (gpointer data, GError **error)
 void
 fill_vtable (GdkPixbufModule *module)
 {
+        /*todo: call this somewhere to make sure the gnome-vfs et. al. are initialized */
+        /*rsvg_init ();*/
+
         module->begin_load     = gdk_pixbuf__svg_image_begin_load;
         module->stop_load      = gdk_pixbuf__svg_image_stop_load;
         module->load_increment = gdk_pixbuf__svg_image_load_increment;
 }
 
+/* this is present only in GTK+ 2.4 and later. we want librsvg to work with older versions too */
+#ifndef GDK_PIXBUF_FORMAT_SCALABLE
+#define GDK_PIXBUF_FORMAT_SCALABLE (1 << 1)
+#endif
+
+/* this is present only in GTK+ 2.6 and later. we want librsvg to work with older versions too. 
+   we can't define this flag yet because Pango isn't threadsafe. */
+#ifndef GDK_PIXBUF_FORMAT_THREADSAFE
+#define GDK_PIXBUF_FORMAT_THREADSAFE (1 << 2)
+#endif
+
 void
 fill_info (GdkPixbufFormat *info)
 {
@@ -174,21 +205,25 @@ fill_info (GdkPixbufFormat *info)
                 { "<!DOCTYPE svg", NULL, 100 },
                 { NULL, NULL, 0 }
         };
-        static gchar *mime_types[] = { 
-                "image/svg", 
+        static gchar *mime_types[] = { /* yes folks, i actually have run into all of these in the wild... */
                 "image/svg+xml",
-                NULL 
+                "image/svg",
+                "image/svg-xml",
+                "image/vnd.adobe.svg+xml",
+                "text/xml-svg",
+                NULL
         };
-        static gchar *extensions[] = { 
-                "svg", 
+        static gchar *extensions[] = {
+                "svg",
                 "svgz",
-                NULL 
+                "svg.gz",
+                NULL
         };
-        
+
         info->name        = "svg";
         info->signature   = signature;
         info->description = _("Scalable Vector Graphics");
         info->mime_types  = mime_types;
         info->extensions  = extensions;
-        info->flags       = 0;
+        info->flags       = GDK_PIXBUF_FORMAT_SCALABLE;
 }
