@@ -39,6 +39,9 @@
 #define MM_PER_INCH     (25.4)
 #define PICA_PER_INCH   (6.0)
 
+#define SETINHERIT() G_STMT_START {if (inherit != NULL) *inherit = TRUE;} G_STMT_END
+#define UNSETINHERIT() G_STMT_START {if (inherit != NULL) *inherit = FALSE;} G_STMT_END
+
 /**
  * rsvg_css_parse_vbox
  * @vbox: The CSS viewBox
@@ -158,7 +161,7 @@ rsvg_css_parse_normalized_length(const char *str, gdouble pixels_per_inch,
 	else if (em)
 		return length * font_size;
 	else if (ex)
-		return length * font_size * 2.0; /* stolen from imagemagick */
+		return (length * font_size) / 2.;
 	else
 		return length;
 }
@@ -168,7 +171,7 @@ rsvg_css_param_match (const char *str, const char *param_name)
 {
 	int i;
 	
-	for (i = 0; str[i] != '\0' && str[i] != ':'; i++)
+	for (i = 0; str[i] != '\0' && param_name[i] != '\0'; i++)
 		if (param_name[i] != str[i])
 			return FALSE;
 	return str[i] == ':' && param_name[i] == '\0';
@@ -178,11 +181,20 @@ int
 rsvg_css_param_arg_offset (const char *str)
 {
 	int i;
-	
-	for (i = 0; str[i] != '\0' && str[i] != ':'; i++);
-	if (str[i] != '\0') i++;
-	for (; str[i] == ' '; i++);
-	return i;
+	int found;
+	found = -1;	
+	i = 0;
+	while (str[i] != '\0')
+		{
+			for (; str[i] != '\0' && str[i] != ':'; i++);
+			if (str[i] != '\0') i++;
+			for (; str[i] == ' '; i++);
+			if (str[i] != '\0')
+				found = i;
+		}
+	if (found == -1)
+		found = i;
+	return found;
 }
 
 static gint
@@ -230,10 +242,12 @@ rsvg_css_color_compare (const void * a, const void * b)
  * Parse a CSS2 color specifier, return RGB value
  */
 guint32
-rsvg_css_parse_color (const char *str, guint32 inherit)
+rsvg_css_parse_color (const char *str, gboolean * inherit)
 {
 	gint val = 0;
-	
+
+	SETINHERIT ();
+
 	if (str[0] == '#')
 		{
 			int i;
@@ -267,7 +281,7 @@ rsvg_css_parse_color (const char *str, guint32 inherit)
 			
 			if (strstr (str, "%") != 0)
 				{
-					gint i, nb_toks;
+					guint i, nb_toks;
 					char ** toks;
 
 					/* assume rgb (9%, 100%, 23%) */
@@ -306,7 +320,7 @@ rsvg_css_parse_color (const char *str, guint32 inherit)
 			val = PACK_RGB (r,g,b);
 		}
 	else if (!strcmp (str, "inherit"))
-		val = inherit;
+		UNSETINHERIT ();
 	else 
 		{
 			const static ColorPair color_list [] =
@@ -365,9 +379,9 @@ rsvg_css_parse_color (const char *str, guint32 inherit)
 					{ "gold",                 PACK_RGB (255,215,0) },
 					{ "goldenrod",            PACK_RGB (218,165,32) },
 					{ "gray",                 PACK_RGB (128,128,128) },
-					{ "grey",                 PACK_RGB (128,128,128) },
-					{ "green",                PACK_RGB (0,128,0)},
+					{ "green",                PACK_RGB (0,128,0) },
 					{ "greenyellow",          PACK_RGB (173,255,47) },
+					{ "grey",                 PACK_RGB (128,128,128) },
 					{ "honeydew",             PACK_RGB (240,255,240) },
 					{ "hotpink",              PACK_RGB (255,105,180) },
 					{ "indianred",            PACK_RGB (205,92,92) },
@@ -407,15 +421,15 @@ rsvg_css_parse_color (const char *str, guint32 inherit)
 					{ "mediumspringgreen",    PACK_RGB (0,250,154) },
 					{ "mediumturquoise",      PACK_RGB (72,209,204) },
 					{ "mediumvioletred",      PACK_RGB (199,21,133) },
-					{ "mediumnightblue",      PACK_RGB (25,25,112) },
+					{ "midnightblue",         PACK_RGB (25,25,112) },
 					{ "mintcream",            PACK_RGB (245,255,250) },
-					{ "mintyrose",            PACK_RGB (255,228,225) },
+					{ "mistyrose",            PACK_RGB (255,228,225) },
 					{ "moccasin",             PACK_RGB (255,228,181) },
 					{ "navajowhite",          PACK_RGB (255,222,173) },
 					{ "navy",                 PACK_RGB (0,0,128) },
 					{ "oldlace",              PACK_RGB (253,245,230) },
 					{ "olive",                PACK_RGB (128,128,0) },
-					{ "oliverab",             PACK_RGB (107,142,35) },
+					{ "olivedrab",            PACK_RGB (107,142,35) },
 					{ "orange",               PACK_RGB (255,165,0) },
 					{ "orangered",            PACK_RGB (255,69,0) },
 					{ "orchid",               PACK_RGB (218,112,214) },
@@ -442,8 +456,8 @@ rsvg_css_parse_color (const char *str, guint32 inherit)
 					{ "silver",               PACK_RGB (192,192,192) },
 					{ "skyblue",              PACK_RGB (135,206,235) },
 					{ "slateblue",            PACK_RGB (106,90,205) },
-					{ "slategray",            PACK_RGB (112,128,144) },
-					{ "slategrey",            PACK_RGB (112,128,114) },
+					{ "slategray",            PACK_RGB (119,128,144) },
+					{ "slategrey",            PACK_RGB (119,128,144) },
 					{ "snow",                 PACK_RGB (255,255,250) },
 					{ "springgreen",          PACK_RGB (0,255,127) },
 					{ "steelblue",            PACK_RGB (70,130,180) },
@@ -467,7 +481,10 @@ rsvg_css_parse_color (const char *str, guint32 inherit)
 			
 			/* default to black on failed lookup */
 			if (result == NULL)
-				val = 0;
+				{
+					UNSETINHERIT ();
+					val = 0;
+				}
 			else
 				val = result->rgb;
 		}
@@ -583,8 +600,10 @@ rsvg_css_parse_time (const char * str)
 }
 
 PangoStyle
-rsvg_css_parse_font_style (const char * str, PangoStyle inherit)
+rsvg_css_parse_font_style (const char * str, gboolean * inherit)
 {
+	SETINHERIT ();
+
 	if (str)
 		{
 			if (!strcmp(str, "oblique"))
@@ -592,27 +611,38 @@ rsvg_css_parse_font_style (const char * str, PangoStyle inherit)
 			if (!strcmp(str, "italic"))
 				return PANGO_STYLE_ITALIC;
 			else if (!strcmp(str, "inherit"))
-				return inherit;
+				{
+					UNSETINHERIT ();
+					return PANGO_STYLE_NORMAL;
+				}
 		}
+	UNSETINHERIT ();
 	return PANGO_STYLE_NORMAL;
 }
 
 PangoVariant
-rsvg_css_parse_font_variant (const char * str, PangoVariant inherit)
+rsvg_css_parse_font_variant (const char * str, gboolean * inherit)
 {
+	SETINHERIT ();
+
 	if (str)
     {
 		if (!strcmp(str, "small-caps"))
 			return PANGO_VARIANT_SMALL_CAPS;
 		else if (!strcmp(str, "inherit"))
-			return inherit;
+			{
+				UNSETINHERIT ();
+				return PANGO_VARIANT_NORMAL;
+			}
     }
+	UNSETINHERIT ();
 	return PANGO_VARIANT_NORMAL;
 }
 
 PangoWeight
-rsvg_css_parse_font_weight (const char * str, PangoWeight inherit)
+rsvg_css_parse_font_weight (const char * str, gboolean * inherit)
 {
+	SETINHERIT ();
 	if (str)
 		{
 			if (!strcmp (str, "lighter"))
@@ -640,15 +670,21 @@ rsvg_css_parse_font_weight (const char * str, PangoWeight inherit)
 			else if (!strcmp (str, "900"))
 				return (PangoWeight)900;
 			else if (!strcmp(str, "inherit"))
-				return inherit;
+				{
+					UNSETINHERIT ();
+					return PANGO_WEIGHT_NORMAL;
+				}
 		}
-	
+
+	UNSETINHERIT ();	
 	return PANGO_WEIGHT_NORMAL; 
 }
 
 PangoStretch
-rsvg_css_parse_font_stretch (const char * str, PangoStretch inherit)
+rsvg_css_parse_font_stretch (const char * str, gboolean * inherit)
 {
+	SETINHERIT ();
+
 	if (str)
 		{
 			if (!strcmp (str, "ultra-condensed"))
@@ -668,23 +704,32 @@ rsvg_css_parse_font_stretch (const char * str, PangoStretch inherit)
 			else if (!strcmp (str, "ultra-expanded"))
 				return PANGO_STRETCH_ULTRA_EXPANDED;
 			else if (!strcmp(str, "inherit"))
-				return inherit;
-		}
+				{
+					UNSETINHERIT ();
+					return PANGO_STRETCH_NORMAL;
+				}
+		}	
+	UNSETINHERIT ();
 	return PANGO_STRETCH_NORMAL;
 }
 
 const char *
-rsvg_css_parse_font_family (const char * str, const char * inherit)
+rsvg_css_parse_font_family (const char * str, gboolean * inherit)
 {
+	SETINHERIT ();
+
 	if (!str)
 		return NULL;	
 	else if (!strcmp (str, "inherit"))
-		return inherit;
+		{
+			UNSETINHERIT ();
+			return NULL;
+		}
 	else
 		return str;
 }
 
-#if !defined(HAVE_STRTOK_R) && !GLIB_CHECK_VERSION(2, 3, 2)
+#if !defined(HAVE_STRTOK_R)
 
 static char *
 strtok_r(char *s, const char *delim, char **last)
@@ -725,23 +770,6 @@ strtok_r(char *s, const char *delim, char **last)
 gchar **
 rsvg_css_parse_list(const char * in_str, guint * out_list_len)
 {
-#if GLIB_CHECK_VERSION(2, 3, 2)
-
-	gchar ** string_array;
-	guint n;
-
-	/* this may fix bug #113538 */
-
-	string_array = g_strsplit_set(in_str, ", \t\n", -1);
-
-	for(n = 0; string_array[n] != NULL; n++)
-		;
-
-	*out_list_len = n;
-	return string_array;
-
-#else
-
 	char *ptr, *tok;
 	char *str;
 
@@ -752,12 +780,18 @@ rsvg_css_parse_list(const char * in_str, guint * out_list_len)
 	str = g_strdup (in_str);
     tok = strtok_r (str, ", \t", &ptr);
 	if (tok != NULL) {
-		string_list = g_slist_prepend(string_list, g_strdup(tok));
-		n++;
+		if (strcmp(tok, " ") != 0)
+			{
+				string_list = g_slist_prepend(string_list, g_strdup(tok));
+				n++;
+			}
 
 		while((tok = strtok_r (NULL, ", \t", &ptr)) != NULL) {
-			string_list = g_slist_prepend(string_list, g_strdup(tok));
-			n++;
+			if (strcmp(tok, " ") != 0)
+				{
+					string_list = g_slist_prepend(string_list, g_strdup(tok));
+					n++;
+				}
 		}
 	}
 	g_free (str);
@@ -778,8 +812,6 @@ rsvg_css_parse_list(const char * in_str, guint * out_list_len)
 	}
 
 	return string_array;
-
-#endif
 }
 
 gdouble *
@@ -801,7 +833,7 @@ rsvg_css_parse_number_list(const char * in_str, guint * out_list_len){
 	/* TODO: some error checking */
 	for (i = 0; i < len; i++)
 		output[i] = g_ascii_strtod(string_array[i], NULL);
-
+	
 	g_strfreev(string_array);
 
 	if (out_list_len != NULL)
@@ -829,3 +861,54 @@ rsvg_css_parse_number_optional_number(const char * str,
   else
     *y = *x;
 }
+
+int 
+rsvg_css_parse_aspect_ratio (const char * str)
+{
+	char ** elems;
+	guint nb_elems;
+
+	int ratio = RSVG_ASPECT_RATIO_NONE;
+
+	elems = rsvg_css_parse_list (str, &nb_elems);
+
+	if (elems && nb_elems)
+		{
+			guint i;
+
+			for (i = 0; i < nb_elems; i++) {
+				if (!strcmp (elems[i], "xMinYMin"))
+					ratio = RSVG_ASPECT_RATIO_XMIN_YMIN;
+				else if (!strcmp (elems[i], "xMidYMin"))
+					ratio = RSVG_ASPECT_RATIO_XMID_YMIN;
+				else if (!strcmp (elems[i], "xMaxYMin"))
+					ratio = RSVG_ASPECT_RATIO_XMAX_YMIN;
+				else if (!strcmp (elems[i], "xMinYMid"))
+					ratio = RSVG_ASPECT_RATIO_XMIN_YMID;
+				else if (!strcmp (elems[i], "xMidYMid"))
+					ratio = RSVG_ASPECT_RATIO_XMID_YMID;
+				else if (!strcmp (elems[i], "xMaxYMid"))
+					ratio = RSVG_ASPECT_RATIO_XMAX_YMID;
+				else if (!strcmp (elems[i], "xMinYMax"))
+					ratio = RSVG_ASPECT_RATIO_XMIN_YMAX;
+				else if (!strcmp (elems[i], "xMidYMax"))
+					ratio = RSVG_ASPECT_RATIO_XMID_YMAX;
+				else if (!strcmp (elems[i], "xMaxYMax"))
+					ratio = RSVG_ASPECT_RATIO_XMAX_YMAX;
+				else if (!strcmp (elems[i], "slice"))
+					ratio |= RSVG_ASPECT_RATIO_SLICE;			
+			}
+
+			g_strfreev (elems);
+		}
+
+	return ratio;
+}
+
+gboolean 
+rsvg_css_parse_overflow(const char * str)
+{
+	if (!strcmp(str, "visible"))
+		return 1;
+	return 0; /* hidden, scroll */
+}
