@@ -44,6 +44,7 @@ typedef enum {
 	RSVG_DEF_FILTER,
 	RSVG_DEF_MASK,
 	RSVG_DEF_MARKER,
+	RSVG_DEF_SYMBOL,
 	RSVG_DEF_CLIP_PATH
 } RsvgDefType;
 
@@ -64,6 +65,9 @@ rsvg_defs_set (RsvgDefs *defs, const char *name, RsvgDefVal *val);
 void
 rsvg_defs_free (RsvgDefs *defs);
 
+void
+rsvg_defs_set_base_uri (RsvgDefs * self, gchar * base_uri);
+
 G_END_DECLS
 
 #endif
