@@ -43,14 +43,14 @@ typedef struct _RsvgPaintServer RsvgPaintServer;
 typedef struct _RsvgPSCtx RsvgPSCtx;
 
 struct _RsvgPSCtx {
-	int x0;
-	int y0;
-	int x1;
-	int y1;
+	double x0;
+	double y0;
+	double x1;
+	double y1;
 
 	guint32 color;
 	double affine[6];
-	RsvgHandle *ctx;
+	DrawingCtx *ctx;
 };
 
 struct _RsvgGradientStop {
@@ -93,15 +93,18 @@ struct _RsvgPattern {
 	RsvgDefVal super;
 	gboolean obj_cbbox;
 	gboolean obj_bbox;
+	gboolean vbox;
 	double affine[6]; /* user space to actual at time of gradient def */
 	double x, y, width, height;
+	double vbx, vby, vbh, vbw;
 	RsvgDefVal * g;
 	RsvgDefVal * gfallback;
+	unsigned int preserve_aspect_ratio;
 };
 
 /* Create a new paint server based on a specification string. */
 RsvgPaintServer *
-rsvg_paint_server_parse (RsvgPaintServer * current, const RsvgDefs *defs, const char *str,
+rsvg_paint_server_parse (gboolean * inherit, const RsvgDefs *defs, const char *str,
 						 guint32 current_color);
 
 void
