@@ -37,6 +37,8 @@
 G_BEGIN_DECLS
 
 typedef struct RsvgSaxHandler RsvgSaxHandler;
+typedef struct DrawingCtx DrawingCtx;
+typedef struct RsvgDimentionData RsvgDimentionData;
 typedef struct _RsvgPropertyBag RsvgPropertyBag;
 typedef struct _RsvgState RsvgState;
 typedef struct _RsvgDefs RsvgDefs;
@@ -48,6 +50,10 @@ typedef struct _RsvgFilter RsvgFilter;
 #define _(X) X
 #endif
 
+#ifndef N_
+#define N_(X) X
+#endif
+
 struct RsvgSaxHandler {
 	void (*free) (RsvgSaxHandler *self);
 	void (*start_element) (RsvgSaxHandler *self, const xmlChar *name, RsvgPropertyBag *atts);
@@ -59,20 +65,18 @@ struct RsvgHandle {
 	RsvgSizeFunc size_func;
 	gpointer user_data;
 	GDestroyNotify user_data_destroy;
-	GdkPixbuf *pixbuf;
 	ArtIRect bbox;
 
 	/* stack; there is a state for each element */
-
-	GSList * state;
 	
 	RsvgDefs *defs;
-	guint in_defs;
 	guint nest_level;
 	void *current_defs_group;
+	/* this is the root level of the displayable tree, essentially what the
+	   file is converted into at the end */
+	void *treebase;
 
 	guint in_switch;
-
 	GHashTable *css_props;
 	
 	/* not a handler stack. each nested handler keeps
@@ -83,25 +87,29 @@ struct RsvgHandle {
 	
 	GHashTable *entities; /* g_malloc'd string -> xmlEntityPtr */
 	
-	PangoContext *pango_context;
 	xmlParserCtxtPtr ctxt;
 	GError **error;
 	
 	int width;
 	int height;
+	int new_width;
+	int new_height;
 	double dpi_x;
 	double dpi_y;
+
+	GSList * dimentions;
 	
 	GString * title;
 	GString * desc;
+	GString * metadata;
 	
 	gchar * base_uri;
 
+	gboolean finished;
+
 	void * currentfilter;
 	void * currentsubfilter;
 
-	GMemChunk * state_allocator;
-
 	/* virtual fns */
 	gboolean (* write) (RsvgHandle    *handle,
 						const guchar  *buf,
@@ -114,6 +122,26 @@ struct RsvgHandle {
 	void (* free) (RsvgHandle * handle);
 };
 
+
+struct DrawingCtx {
+	GdkPixbuf *pixbuf;
+	ArtIRect bbox;
+	GSList * state;
+	GError **error;
+	RsvgDefs *defs;
+	gchar * base_uri;
+	GMemChunk * state_allocator;
+	PangoContext *pango_context;
+	double dpi_x;
+	double dpi_y;
+};
+
+struct RsvgDimentionData {
+	int width;
+	int height;
+	gdouble em, ex;
+};
+
 void rsvg_linear_gradient_free (RsvgDefVal *self);
 void rsvg_radial_gradient_free (RsvgDefVal *self);
 void rsvg_pattern_free (RsvgDefVal *self);
@@ -155,6 +183,10 @@ struct _RsvgPropertyBag
 	GHashTable * props;
 };
 
+typedef void (*RsvgPropertyBagEnumFunc) (const char * key,
+										 const char * value,
+										 gpointer user_data);
+
 RsvgPropertyBag *
 rsvg_property_bag_new (const xmlChar **atts);
 
@@ -167,6 +199,9 @@ rsvg_property_bag_lookup (RsvgPropertyBag *bag, const char * key);
 guint
 rsvg_property_bag_size (RsvgPropertyBag *bag);
 
+void 
+rsvg_property_bag_enumerate (RsvgPropertyBag * bag, RsvgPropertyBagEnumFunc func, gpointer user_data);
+
 GdkPixbuf *
 rsvg_pixbuf_from_data_with_size_data (const guchar * buff,
 									  size_t len,
@@ -174,12 +209,6 @@ rsvg_pixbuf_from_data_with_size_data (const guchar * buff,
 									  const char * base_uri,
 									  GError ** error);
 
-G_CONST_RETURN char *
-rsvg_handle_get_base_uri (RsvgHandle *handle);
-
-void rsvg_handle_set_base_uri (RsvgHandle *handle,
-							   const char *base_uri);
-
 gboolean 
 rsvg_eval_switch_attributes (RsvgPropertyBag *atts, gboolean * p_has_cond);
 
@@ -187,6 +216,13 @@ GdkPixbuf *
 _rsvg_pixbuf_new_cleared (GdkColorspace colorspace, gboolean has_alpha, int bits_per_sample,
 						  int width, int height);
 
+gchar *
+rsvg_get_base_uri_from_filename(const gchar * file_name);
+
+GByteArray *
+_rsvg_acquire_xlink_href_resource (const char *href,
+								   const char *base_uri,
+								   GError **err);
 G_END_DECLS
 
 #endif
