@@ -66,11 +66,11 @@ struct _RsvgState {
 	double affine[6];
 	double personal_affine[6];
 	
-	gint opacity; /* 0..255 */
+	guint8 opacity; /* 0..255 */
 	
 	RsvgPaintServer *fill;
 	gboolean has_fill_server;
-	gint fill_opacity; /* 0..255 */
+	guint8 fill_opacity; /* 0..255 */
 	gboolean has_fill_opacity;
 	gint fill_rule;	
 	gboolean has_fill_rule;
@@ -81,11 +81,11 @@ struct _RsvgState {
 	void *mask;
 	void *clip_path_ref;
 	gboolean backgroundnew;
-	gint adobe_blend;
+	guint8 adobe_blend; /* 0..11 */
 
 	RsvgPaintServer *stroke;
 	gboolean has_stroke_server;
-	gint stroke_opacity; /* 0..255 */
+	guint8 stroke_opacity; /* 0..255 */
 	gboolean has_stroke_opacity;
 	double stroke_width;
 	gboolean has_stroke_width;
@@ -150,9 +150,10 @@ struct _RsvgState {
 	ArtIRect underbbox;
 
 	ArtSVP * clippath;
-	gboolean clip_path_loaded;
+	int clip_path_loaded : 1;
 };
 
+RsvgState * rsvg_state_new ();
 void rsvg_state_init (RsvgState *state);
 void rsvg_state_clone (RsvgState *dst, const RsvgState *src);
 void rsvg_state_inherit (RsvgState *dst, const RsvgState *src);
@@ -174,21 +175,21 @@ void rsvg_parse_style_attrs (RsvgHandle *ctx, RsvgState *state, const char * tag
 gdouble rsvg_viewport_percentage (gdouble width, gdouble height);
 gdouble rsvg_dpi_percentage (RsvgHandle * ctx);
 
-void rsvg_pop_discrete_layer(RsvgHandle *ctx);
-void rsvg_push_discrete_layer (RsvgHandle *ctx);
+void rsvg_pop_discrete_layer(DrawingCtx *ctx);
+void rsvg_push_discrete_layer (DrawingCtx *ctx);
 gboolean rsvg_needs_discrete_layer(RsvgState *state);
 gboolean rsvg_parse_transform (double dst[6], const char *src);
 
-RsvgState * rsvg_state_parent (RsvgHandle *ctx);
-RsvgState * rsvg_state_current (RsvgHandle *ctx);
+RsvgState * rsvg_state_parent (DrawingCtx *ctx);
+RsvgState * rsvg_state_current (DrawingCtx *ctx);
 double rsvg_state_current_font_size (RsvgHandle *ctx);
 
-void rsvg_state_clip_path_assure(RsvgHandle * ctx);
+void rsvg_state_clip_path_assure(DrawingCtx * ctx);
 
-void rsvg_state_pop(RsvgHandle * ctx);
-void rsvg_state_push(RsvgHandle * ctx);
+void rsvg_state_pop(DrawingCtx * ctx);
+void rsvg_state_push(DrawingCtx * ctx);
 
-void rsvg_state_reinherit_top(RsvgHandle * ctx, RsvgState * state, int dominate);
+void rsvg_state_reinherit_top(DrawingCtx * ctx, RsvgState * state, int dominate);
 
 G_END_DECLS
 
