@@ -27,11 +27,14 @@
 
 #include "rsvg-private.h"
 #include "rsvg-styles.h"
+#include "rsvg-shapes.h"
 #include "rsvg-filter.h"
 #include "rsvg-text.h"
 #include "rsvg-css.h"
+#include "rsvg-mask.h"
 
 #include <libart_lgpl/art_affine.h>
+#include <libart_lgpl/art_svp_ops.h>
 #include <libart_lgpl/art_render_mask.h>
 
 #include "rsvg-shapes.h"
@@ -79,16 +82,65 @@ rsvg_make_valid_utf8 (const char *str)
 
 typedef struct _RsvgTspan RsvgTspan;
 
+typedef struct _RsvgTChunk RsvgTChunk;
+
 struct _RsvgTspan {
 	gdouble x, y;
 	gboolean hasx, hasy;
 	gdouble dx, dy;
 	RsvgTspan * parent;
+	gint parentindex;
+	GPtrArray * contents;
+	RsvgState state;
 };
 
+struct _RsvgTChunk {
+	GString * string;
+	RsvgTspan * span;
+};
+
+static RsvgTChunk *
+rsvg_tchunk_new_text(gchar *text)
+{
+	RsvgTChunk * output;
+	output = g_new(RsvgTChunk, 1);
+	output->string = g_string_new(text);
+	output->span = NULL;
+	return output;
+}
+
+static RsvgTChunk *
+rsvg_tchunk_new_span(RsvgTspan *span)
+{
+	RsvgTChunk * output;
+	output = g_new(RsvgTChunk, 1);
+	output->string = NULL;
+	output->span = span;
+	return output;
+}
+
+static void
+rsvg_tchunk_free(RsvgTChunk * self);
+
 static void
 rsvg_tspan_free(RsvgTspan * self)
 {
+	unsigned int i;
+	rsvg_state_finalize(&self->state);
+	for (i = 0; i < self->contents->len; i++) {
+		rsvg_tchunk_free (g_ptr_array_index(self->contents, i));
+	}
+	g_ptr_array_free(self->contents, 1);
+	g_free(self);
+}
+
+static void
+rsvg_tchunk_free(RsvgTChunk * self)
+{
+	if (self->string)
+		g_string_free(self->string, 1);
+	if (self->span)
+		rsvg_tspan_free(self->span);
 	g_free(self);
 }
 
@@ -101,70 +153,57 @@ rsvg_tspan_new()
 	self->dy = 0;
 	self->hasx = FALSE;
 	self->hasy = FALSE;
+	self->parent = NULL;
+	self->parentindex = 0;
+	self->contents = g_ptr_array_new();
 	return self;
 }
 
-static gdouble
-rsvg_tspan_x(RsvgTspan * self)
-{
-	RsvgTspan * i;
-	for (i = self; i->hasx == FALSE; i = i->parent)
-		;
-	return i->x;
-}
+static void
+rsvg_tchunk_remove_leading(RsvgTChunk * self);
 
-static gdouble
-rsvg_tspan_y(RsvgTspan * self)
+static void
+rsvg_tspan_remove_leading(RsvgTspan * self)
 {
-	RsvgTspan * i;
-	for (i = self; i->hasy == FALSE; i = i->parent)
-		;
-	return i->y;
+	if (!self)
+		return;
+	if (!self->contents->len == 0)
+		return
+	rsvg_tchunk_remove_leading(g_ptr_array_index(self->contents, 0));
 }
 
 static void
-rsvg_tspan_set_x(RsvgTspan * self, gdouble setto)
+rsvg_tchunk_remove_leading(RsvgTChunk * self)
 {
-	RsvgTspan * i;
-	for (i = self; i->hasx == FALSE; i = i->parent)
-		;
-	i->x = setto;
+	if (self->string)
+		if (self->string->str[0] == ' ')
+			g_string_erase(self->string, 0, 1);
+	if (self->span)
+		rsvg_tspan_remove_leading(self->span);
 }
-/*
+
 static void
-rsvg_tspan_set_y(RsvgTspan * self, gdouble setto)
-{
-	RsvgTspan * i;
-	for (i = self; i->hasy == FALSE; i = i->parent)
-		;
-	i->y = setto;
-}
-*/
+rsvg_tchunk_remove_trailing(RsvgTChunk * self);
 
-static gdouble
-rsvg_tspan_dx(RsvgTspan * self)
+static void
+rsvg_tspan_remove_trailing(RsvgTspan * self)
 {
-	RsvgTspan * i;
-	gdouble total;
-
-	total = 0;
-	for (i = self; i->hasx == FALSE; i = i->parent)
-		total += i->dx;
-	total += i->dx;
-	return total;
+	if (!self)
+		return;
+	if (!self->contents->len == 0)
+		return
+	rsvg_tchunk_remove_trailing(g_ptr_array_index(self->contents, 
+												  self->contents->len - 1));
 }
 
-static gdouble
-rsvg_tspan_dy(RsvgTspan * self)
+static void
+rsvg_tchunk_remove_trailing(RsvgTChunk * self)
 {
-	RsvgTspan * i;
-	gdouble total;
-
-	total = 0;
-	for (i = self; i->hasy == FALSE; i = i->parent)
-		total += i->dy;
-	total += i->dy;
-	return total;
+	if (self->string)
+		if (self->string->str[self->string->len] == ' ')
+			g_string_erase(self->string, self->string->len - 1, 1);
+	if (self->span)
+		rsvg_tspan_remove_trailing(self->span);
 }
 
 typedef struct _RsvgSaxHandlerText {
@@ -174,6 +213,7 @@ typedef struct _RsvgSaxHandlerText {
 	GString * id;
 	RsvgTspan * tspan;
 	RsvgTspan * innerspan;
+	RsvgDefsDrawableText * block;
 } RsvgSaxHandlerText;
 
 static void
@@ -182,56 +222,63 @@ rsvg_text_handler_free (RsvgSaxHandler *self)
 	RsvgSaxHandlerText * z;
 	z = (RsvgSaxHandlerText *)self;
 
-	rsvg_tspan_free(z->tspan);
+	/*maybe this isn't the best place for this*/
+	rsvg_tspan_remove_leading(z->tspan);
+	rsvg_tspan_remove_trailing(z->tspan);
+
 	g_string_free(z->id, TRUE);
 	g_free (self);
 }
 
-void 
-rsvg_text_render_text (RsvgSaxHandlerText *ctx,
-					   RsvgState  *state,
-					   const char *text,
-					   const char *id);
-
 static void
 rsvg_text_handler_characters (RsvgSaxHandler *self, const xmlChar *ch, int len)
 {
-	RsvgSaxHandlerText *z = (RsvgSaxHandlerText *)self;
-	RsvgHandle *ctx = z->ctx;
 	char *string, *tmp;
-	int beg, end;
-	RsvgState *state;
-	
-	state = rsvg_state_current (ctx);
-	if (state->fill == NULL && state->font_size <= 0)
-		return;
+	int i, j;
+	RsvgSaxHandlerText * z;
+	RsvgTChunk * tchunk;
 
-	/* not quite up to spec, but good enough */
-	if (!state->visible)
-		return;
-	
-	/* Copy ch into string, chopping off leading and trailing whitespace */
-	for (beg = 0; beg < len; beg++)
-		if (!g_ascii_isspace (ch[beg]))
-			break;
-	
-	for (end = len; end > beg; end--)
-		if (!g_ascii_isspace (ch[end - 1]))
-			break;
-	
-	if (end - beg == 0)
+	z = (RsvgSaxHandlerText *)self;	
+
+	string = g_try_malloc (len + 1);
+
+	for (i = 0; i < len; i++)
 		{
-			/* TODO: be smarter with some "last was space" logic */
-			end = 1; beg = 0;
-			string = g_strdup (" ");
+			
+			if (ch[i] == '\n' || ch[i] == '\t')
+				string[i] = ' ';
+			else
+				string[i] = ch[i];
+		}
+
+	if (1 /*todo replace with something about xml:space*/)
+		{
+			tmp = g_try_malloc (len + 1);
+			j = 0;
+			for (i = 0; i < len; i++)
+				{
+					if (j == 0)					
+						tmp[j++] = string[i];
+					else
+						{
+							if (string[i] != ' ' || string[i - 1] != ' ')
+								tmp[j++] = string[i];
+						}
+				}
+			tmp[j] = '\0';
+			g_free (string);
+			string = tmp;
 		}
 	else
-		{
-			string = g_malloc (end - beg + 1);
-			memcpy (string, ch + beg, end - beg);
-			string[end - beg] = 0;
-		}
+		j = len;
 	
+	if (j == 0)
+		{
+			g_free (string);
+			return;
+		}
+
+
 	if (!g_utf8_validate (string, -1, NULL))
 		{
 			tmp = rsvg_make_valid_utf8 (string);
@@ -239,7 +286,9 @@ rsvg_text_handler_characters (RsvgSaxHandler *self, const xmlChar *ch, int len)
 			string = tmp;
 		}
 
-	rsvg_text_render_text (z, state, string, z->id->str);
+	tchunk = rsvg_tchunk_new_text(string);
+
+	g_ptr_array_add (z->innerspan->contents, tchunk);
 
 	g_free (string);
 }
@@ -247,57 +296,61 @@ rsvg_text_handler_characters (RsvgSaxHandler *self, const xmlChar *ch, int len)
 static void
 rsvg_start_tspan (RsvgSaxHandlerText *self, RsvgPropertyBag *atts)
 {
-	RsvgState *state;
 	RsvgHandle *ctx;
+	RsvgState state;
 	RsvgSaxHandlerText *z;
 	RsvgTspan * tspan;
+	RsvgTChunk * tchunk;
 	const char * klazz = NULL, * id = NULL, *value;
-
+	double font_size;
 	tspan = rsvg_tspan_new();
 	z = (RsvgSaxHandlerText *)self;
 	ctx = z->ctx;
-	state = rsvg_state_current (ctx);
+	font_size = rsvg_state_current_font_size(ctx);
+	rsvg_state_init(&state);
 
 	if (rsvg_property_bag_size (atts))
 		{
 			if ((value = rsvg_property_bag_lookup (atts, "x")))
 				{
-					tspan->x = rsvg_css_parse_normalized_length (value, ctx->dpi_x, (gdouble)ctx->width, state->font_size);
+					tspan->x = rsvg_css_parse_normalized_length (value, ctx->dpi_x, (gdouble)ctx->width, font_size);
 					tspan->hasx = TRUE;
 				}
 			if ((value = rsvg_property_bag_lookup (atts, "y")))
 				{
-					tspan->y = rsvg_css_parse_normalized_length (value, ctx->dpi_y, (gdouble)ctx->height, state->font_size);
+					tspan->y = rsvg_css_parse_normalized_length (value, ctx->dpi_y, (gdouble)ctx->height, font_size);
 					tspan->hasy = TRUE;
 				}
 			if ((value = rsvg_property_bag_lookup (atts, "dx")))
-				tspan->dx = rsvg_css_parse_normalized_length (value, ctx->dpi_x, (gdouble)ctx->width, state->font_size);
+				tspan->dx = rsvg_css_parse_normalized_length (value, ctx->dpi_x, (gdouble)ctx->width, font_size);
 			if ((value = rsvg_property_bag_lookup (atts, "dy")))
-				tspan->dy = rsvg_css_parse_normalized_length (value, ctx->dpi_y, (gdouble)ctx->height, state->font_size);
+				tspan->dy = rsvg_css_parse_normalized_length (value, ctx->dpi_y, (gdouble)ctx->height, font_size);
 			if ((value = rsvg_property_bag_lookup (atts, "class")))
 				klazz = value;
 			if ((value = rsvg_property_bag_lookup (atts, "id")))
 				id = value;
 
-			rsvg_parse_style_attrs (ctx, state, "tspan", klazz, id, atts);
+			rsvg_parse_style_attrs (ctx, &state, "tspan", klazz, id, atts);
 		}
-	
+
+	tchunk = rsvg_tchunk_new_span(tspan);
+
+	tspan->parentindex = z->innerspan->contents->len;
 	tspan->parent = z->innerspan;
+	tspan->state = state;
+	g_ptr_array_add (z->innerspan->contents, tchunk);
 	z->innerspan = tspan;
+	
 }
 
 static void
 rsvg_text_handler_start (RsvgSaxHandler *self, const xmlChar *name,
 						 RsvgPropertyBag *atts)
 {
-	RsvgSaxHandlerText *z = (RsvgSaxHandlerText *)self;
-	RsvgHandle *ctx = z->ctx;
-
-	rsvg_state_push(ctx);
-  
-	/* this should be the only thing starting inside of text */
+	/*this should be the only thing starting inside of text*/ 
 	if (!strcmp ((char *)name, "tspan"))
 		rsvg_start_tspan ((RsvgSaxHandlerText *)self, atts);
+
 }
 
 static void
@@ -305,13 +358,12 @@ rsvg_text_handler_end (RsvgSaxHandler *self, const xmlChar *name)
 {
 	RsvgSaxHandlerText *z = (RsvgSaxHandlerText *)self;
 	RsvgHandle *ctx = z->ctx;
-
+	
 	if (!strcmp ((char *)name, "tspan"))
 		{
 			RsvgTspan * child;
 			child = z->innerspan;
 			z->innerspan = child->parent;
-			rsvg_tspan_free(child);
 		}
 	else if (!strcmp ((char *)name, "text"))
 		{
@@ -322,49 +374,259 @@ rsvg_text_handler_end (RsvgSaxHandler *self, const xmlChar *name)
 				}
 		} 
 	
-	rsvg_state_pop(ctx);
+}
+
+static void 
+rsvg_defs_drawable_text_free (RsvgDefVal *self)
+{
+	RsvgDefsDrawableText *z = (RsvgDefsDrawableText *)self;
+	rsvg_tspan_free (z->chunk);
+	g_free (z);
+}
+
+void 
+rsvg_text_render_text (DrawingCtx *ctx,
+					   RsvgTspan  *tspan,
+					   const char *text,
+					   gdouble *x,
+					   gdouble *y);
+
+ArtSVP *
+rsvg_text_render_text_as_svp (DrawingCtx *ctx,
+							  RsvgTspan  *tspan,
+							  const char *text,
+							  gdouble *x,
+							  gdouble *y);
+
+static gdouble
+rsvg_text_width  (DrawingCtx *ctx,
+				  RsvgTspan  *tspan,
+				  const char *text);
+
+static gdouble
+rsvg_text_tspan_width (DrawingCtx *ctx,
+					   RsvgTspan  *tspan)
+{
+	RsvgTspan  *currentspan = tspan;
+	gdouble currentwidth = 0;
+	guint currentindex = 0;
+	while (1)
+		{
+			if (currentindex >= currentspan->contents->len)
+				{
+					currentindex = currentspan->parentindex;
+					currentspan = currentspan->parent;
+					if (currentspan == NULL)
+						return currentwidth;
+				}
+			else
+				{
+					RsvgTChunk * currentchunk = 
+						g_ptr_array_index(currentspan->contents, currentindex);
+					if (currentchunk->string)
+						currentwidth += rsvg_text_width (ctx, currentspan, currentchunk->string->str);
+					else
+						{
+							currentspan = currentchunk->span;
+							currentindex = -1;
+							if (currentspan->hasx || currentspan->hasy)
+								return currentwidth;
+							currentwidth += currentspan->dx;
+						}
+				}
+			currentindex++;
+		}
+}
+
+static void
+rsvg_tspan_draw(RsvgTspan * self, DrawingCtx *ctx, gdouble *x, gdouble *y, int dominate);
+
+static void
+rsvg_tchunk_draw(RsvgTChunk * self, DrawingCtx *ctx, RsvgTspan *span, gdouble *x, gdouble *y)
+{
+	if (self->string)
+		rsvg_text_render_text (ctx, span, self->string->str, x, y);
+	if (self->span)
+		{
+			rsvg_state_push(ctx);
+			rsvg_tspan_draw (self->span, ctx, x, y, 0);
+			rsvg_state_pop(ctx);
+		}
+}
+
+static void
+rsvg_tspan_draw(RsvgTspan * self, DrawingCtx *ctx, gdouble *x, gdouble *y, int dominate)
+{
+	unsigned int i;
+	rsvg_state_reinherit_top(ctx, &self->state, dominate);
+	if (self->hasx || self->hasy)
+		{
+			switch (rsvg_state_current(ctx)->text_anchor)
+				{
+				case TEXT_ANCHOR_START:
+					*x = self->x;		
+					break;
+				case TEXT_ANCHOR_MIDDLE:
+					*x = self->x - rsvg_text_tspan_width (ctx, self) / 2;
+					break;
+				case TEXT_ANCHOR_END:
+					*x = self->x - rsvg_text_tspan_width (ctx, self);
+					break;
+				}
+			*y = self->y;
+		}
+
+	if (rsvg_state_current(ctx)->text_dir == PANGO_DIRECTION_TTB_LTR || 
+		rsvg_state_current(ctx)->text_dir == PANGO_DIRECTION_TTB_RTL)
+		{
+			*y += self->dx;
+			*x += self->dy;
+		}
+	else
+		{
+			*x += self->dx;
+			*y += self->dy;
+		}
+	for (i = 0; i < self->contents->len; i++) {
+		rsvg_tchunk_draw (g_ptr_array_index(self->contents, i), ctx, self, x, y);
+	}
+}
+
+static ArtSVP *
+rsvg_tspan_draw_as_svp(RsvgTspan * self, DrawingCtx *ctx, gdouble *x, gdouble *y, int dominate);
+
+static ArtSVP *
+rsvg_tchunk_draw_as_svp(RsvgTChunk * self, DrawingCtx *ctx, RsvgTspan *span, gdouble *x, gdouble *y)
+{
+	if (self->string)
+		return rsvg_text_render_text_as_svp (ctx, span, self->string->str, x, y);
+	if (self->span)
+		{
+			ArtSVP * output;
+			rsvg_state_push(ctx);
+			output = rsvg_tspan_draw_as_svp (self->span, ctx, x, y, 0);
+			rsvg_state_pop(ctx);
+			return output;
+		}
+	return NULL;
+}
+
+/*todo: remove unnesicary code duplication*/
+
+static ArtSVP *
+rsvg_tspan_draw_as_svp(RsvgTspan * self, DrawingCtx *ctx, gdouble *x, gdouble *y, int dominate)
+{
+	unsigned int i;
+	ArtSVP *svp1, *svp2, *svp3;
+	svp1 = NULL;
+
+	rsvg_state_reinherit_top(ctx, &self->state, dominate);
+	if (self->hasx || self->hasy)
+		{
+			switch (rsvg_state_current(ctx)->text_anchor)
+				{
+				case TEXT_ANCHOR_START:
+					*x = self->x;		
+					break;
+				case TEXT_ANCHOR_MIDDLE:
+					*x = self->x - rsvg_text_tspan_width (ctx, self) / 2;
+					break;
+				case TEXT_ANCHOR_END:
+					*x = self->x - rsvg_text_tspan_width (ctx, self);
+					break;
+				}
+			*y = self->y;
+		}
+
+	for (i = 0; i < self->contents->len; i++) {
+		svp2 = rsvg_tchunk_draw_as_svp (g_ptr_array_index(self->contents, i), ctx, self, x, y);
+		if (svp1 != NULL)
+			{
+				svp3 = art_svp_union(svp2, svp1);
+				art_free(svp1);
+				svp1 = svp3;
+			}
+		else
+			svp1 = svp2;
+	}
+	return svp1;
+}
+
+
+static void 
+rsvg_defs_drawable_text_draw (RsvgDefsDrawable * self, DrawingCtx *ctx, 
+							  int dominate)
+{
+	gdouble x, y;
+	RsvgDefsDrawableText *text = (RsvgDefsDrawableText*)self;
+
+	rsvg_tspan_draw(text->chunk, ctx, &x, &y, dominate);
+}
+
+static ArtSVP *
+rsvg_defs_drawable_text_draw_as_svp (RsvgDefsDrawable * self, DrawingCtx *ctx, 
+									 int dominate)
+{
+	gdouble x, y;
+	RsvgDefsDrawableText *text = (RsvgDefsDrawableText*)self;
+
+	return rsvg_tspan_draw_as_svp(text->chunk, ctx, &x, &y, dominate);
 }
 
 void
 rsvg_start_text (RsvgHandle *ctx, RsvgPropertyBag *atts)
 {
-	double x, y, dx, dy;
+	double x, y, dx, dy, font_size;
 	const char * klazz = NULL, * id = NULL, *value;
-	RsvgState *state;
-	
+	RsvgState state;
+	RsvgDefsDrawableText *text;
 	RsvgSaxHandlerText *handler = g_new0 (RsvgSaxHandlerText, 1);
+
 	handler->super.free = rsvg_text_handler_free;
 	handler->super.characters = rsvg_text_handler_characters;
 	handler->super.start_element = rsvg_text_handler_start;
 	handler->super.end_element   = rsvg_text_handler_end;
 	handler->ctx = ctx;
-	
+	font_size = rsvg_state_current_font_size(ctx);
 	x = y = dx = dy = 0.;
 	
-	state = rsvg_state_current (ctx);
+	rsvg_state_init(&state);
 	
 	if (rsvg_property_bag_size (atts))
 		{
 			if ((value = rsvg_property_bag_lookup (atts, "x")))
-				x = rsvg_css_parse_normalized_length (value, ctx->dpi_x, (gdouble)ctx->width, state->font_size);
+				x = rsvg_css_parse_normalized_length (value, ctx->dpi_x, (gdouble)ctx->width, font_size);
 			if ((value = rsvg_property_bag_lookup (atts, "y")))
-				y = rsvg_css_parse_normalized_length (value, ctx->dpi_y, (gdouble)ctx->height, state->font_size);
+				y = rsvg_css_parse_normalized_length (value, ctx->dpi_y, (gdouble)ctx->height, font_size);
 			if ((value = rsvg_property_bag_lookup (atts, "dx")))
-				dx = rsvg_css_parse_normalized_length (value, ctx->dpi_x, (gdouble)ctx->width, state->font_size);
+				dx = rsvg_css_parse_normalized_length (value, ctx->dpi_x, (gdouble)ctx->width, font_size);
 			if ((value = rsvg_property_bag_lookup (atts, "dy")))
-				dy = rsvg_css_parse_normalized_length (value, ctx->dpi_y, (gdouble)ctx->height, state->font_size);
+				dy = rsvg_css_parse_normalized_length (value, ctx->dpi_y, (gdouble)ctx->height, font_size);
 			if ((value = rsvg_property_bag_lookup (atts, "class")))
 				klazz = value;
 			if ((value = rsvg_property_bag_lookup (atts, "id")))
 				id = value;
 
-			rsvg_parse_style_attrs (ctx, state, "text", klazz, id, atts);
+			rsvg_parse_style_attrs (ctx, &state, "text", klazz, id, atts);
 		}
 	
+	text = g_new (RsvgDefsDrawableText, 1);
+
+	text->super.super.type = RSVG_DEF_PATH;
+	text->super.super.free = rsvg_defs_drawable_text_free;
+	text->super.draw = rsvg_defs_drawable_text_draw;
+	text->super.draw_as_svp = rsvg_defs_drawable_text_draw_as_svp;
+	rsvg_defs_set (ctx->defs, id, &text->super.super);
+	
+	text->super.parent = (RsvgDefsDrawable *)ctx->current_defs_group;
+	if (text->super.parent != NULL)
+		rsvg_defs_drawable_group_pack((RsvgDefsDrawableGroup *)text->super.parent, 
+									  &text->super);
+
 	handler->id = g_string_new(id);
 
 	handler->tspan = rsvg_tspan_new();
-
+	handler->tspan->parent = NULL;
 	handler->tspan->x = x;
 	handler->tspan->y = y;
 	handler->tspan->hasx = TRUE;
@@ -374,9 +636,12 @@ rsvg_start_text (RsvgHandle *ctx, RsvgPropertyBag *atts)
 	handler->tspan->dy = dy;
 
 	handler->innerspan = handler->tspan;
+	handler->block = text;
+	handler->tspan->state = state;
 
 	handler->parent = ctx->handler;
 	ctx->handler = &handler->super;
+	text->chunk = handler->tspan;
 }
 
 typedef struct _RsvgTextLayout RsvgTextLayout;
@@ -384,9 +649,11 @@ typedef struct _RsvgTextLayout RsvgTextLayout;
 struct _RsvgTextLayout
 {
 	PangoLayout * layout;
-	RsvgHandle  * ctx;
-	RsvgSaxHandlerText * th;
+	DrawingCtx  * ctx;
 	TextAnchor anchor;
+	RsvgTspan    *span;
+	gdouble x, y;
+	gboolean orientation;
 };
 
 typedef struct _RenderCtx RenderCtx;
@@ -447,7 +714,7 @@ rsvg_text_ft2_subst_func (FcPattern *pattern,
 }
 
 static PangoContext *
-rsvg_text_get_pango_context (RsvgHandle *ctx)
+rsvg_text_get_pango_context (DrawingCtx *ctx)
 {
 	PangoContext    *context;
 	PangoFT2FontMap *fontmap;
@@ -475,13 +742,13 @@ rsvg_text_layout_free(RsvgTextLayout * layout)
 }
 
 static RsvgTextLayout *
-rsvg_text_layout_new (RsvgHandle *ctx,
+rsvg_text_layout_new (DrawingCtx *ctx,
 					  RsvgState *state,
 					  const char *text)
 {
 	RsvgTextLayout * layout;
 	PangoFontDescription *font_desc;
-	
+
 	if (ctx->pango_context == NULL)
 		ctx->pango_context = rsvg_text_get_pango_context (ctx);
 	
@@ -684,7 +951,7 @@ cubicto (FT_Vector *ftcontrol1,
 	return 0;
 }
 
-static void
+static gint
 rsvg_text_layout_render_glyphs (RsvgTextLayout     *layout,
 								PangoFont          *font,
 								PangoGlyphString   *glyphs,
@@ -712,9 +979,10 @@ rsvg_text_layout_render_glyphs (RsvgTextLayout     *layout,
 								 pos.x, pos.y,
 								 render_data);
 				}
-			
+		  
 			x_position += glyphs->glyphs[i].geometry.width;
 		}
+	return x_position;
 }
 
 static void
@@ -777,13 +1045,12 @@ rsvg_text_layout_render_line (RsvgTextLayout     *layout,
 			
 			pango_glyph_string_extents (run->glyphs, run->item->analysis.font,
 										NULL, &rect);
-			rsvg_text_layout_render_glyphs (layout,
-											run->item->analysis.font, run->glyphs,
-											render_func,
-											x + x_off, y,
-											render_data);
+			x_off += rsvg_text_layout_render_glyphs (layout,
+													 run->item->analysis.font, run->glyphs,
+													 render_func,
+													 x + x_off, y,
+													 render_data);
 			
-			x_off += rect.width;
 		}
 }
 
@@ -795,25 +1062,18 @@ rsvg_text_layout_render (RsvgTextLayout     *layout,
 	PangoLayoutIter *iter;
 	gint             offx, offy;
 	gint             x, y;
-	gdouble          xshift;
-	gint anchoroffset;
-
-	xshift = 0;
 
 	rsvg_text_layout_get_offsets (layout, &offx, &offy);
 
-	offx += rsvg_tspan_dx(layout->th->innerspan);
-	offy += rsvg_tspan_dy(layout->th->innerspan);
-
-	x = offx + rsvg_tspan_x(layout->th->innerspan);
-	y = offy + rsvg_tspan_y(layout->th->innerspan);
+	x = offx + layout->x;
+	y = offy + layout->y;
 
 	x *= PANGO_SCALE;
 	y *= PANGO_SCALE;
 	
 	iter = pango_layout_get_iter (layout->layout);
 	
-	do
+	if (iter)
 		{
 			PangoRectangle   rect;
 			PangoLayoutLine *line;
@@ -824,43 +1084,40 @@ rsvg_text_layout_render (RsvgTextLayout     *layout,
 			pango_layout_iter_get_line_extents (iter, NULL, &rect);
 			baseline = pango_layout_iter_get_baseline (iter);
 			
-			if (layout->anchor == TEXT_ANCHOR_START)
-				anchoroffset = 0;
-			else if (layout->anchor == TEXT_ANCHOR_MIDDLE)
-				anchoroffset = rect.width / 2;
-			else
-				anchoroffset = rect.width;
-			
 			rsvg_text_layout_render_line (layout, line,
 										  render_func,
-										  x + rect.x - anchoroffset,
+										  x + rect.x,
 										  y + baseline,
 										  render_data);
-			xshift += rect.width;
+
+			layout->x += rect.width / PANGO_SCALE + offx;
 		}
-	while (pango_layout_iter_next_line (iter));
-	
+
 	pango_layout_iter_free (iter);
-
-	rsvg_tspan_set_x(layout->th->innerspan, (x + xshift) / PANGO_SCALE);
-
 }
 
-void 
-rsvg_text_render_text (RsvgSaxHandlerText *self,
-					   RsvgState  *state,
-					   const char *text,
-					   const char *id)
+static GString * 
+rsvg_text_render_text_as_string (DrawingCtx *ctx,
+								 RsvgTspan  *tspan,
+								 const char *text,
+								 gdouble *x,
+								 gdouble *y)
 {
 	RsvgTextLayout *layout;
 	RenderCtx      *render;
-	RsvgHandle     *ctx = self->ctx;
+	RsvgState      *state;
+	GString        *output;
+	state = rsvg_state_current(ctx);
 
 	state->fill_rule = FILL_RULE_EVENODD;	
 	state->has_fill_rule = TRUE;
 
 	layout = rsvg_text_layout_new (ctx, state, text);
-	layout->th = self;
+	layout->span = tspan;
+	layout->x = *x;
+	layout->y = *y;
+	layout->orientation = rsvg_state_current(ctx)->text_dir == PANGO_DIRECTION_TTB_LTR || 
+		rsvg_state_current(ctx)->text_dir == PANGO_DIRECTION_TTB_RTL;
 	render = rsvg_render_ctx_new ();
 
 	rsvg_text_layout_render (layout, rsvg_text_render_vectors, 
@@ -869,12 +1126,85 @@ rsvg_text_render_text (RsvgSaxHandlerText *self,
 	if (render->wrote)
 		g_string_append_c(render->path, 'Z');
 
-#ifdef RSVG_TEXT_DEBUG
-	fprintf(stderr, "%s\n", render->path->str);
-#endif
-
-	rsvg_handle_path (ctx, render->path->str, id);
+	*x = layout->x;
+	*y = layout->y;
 
+	output = g_string_new(render->path->str);
 	rsvg_render_ctx_free (render);
 	rsvg_text_layout_free (layout);
+	return output;
+}
+
+void
+rsvg_text_render_text (DrawingCtx *ctx,
+					   RsvgTspan  *tspan,
+					   const char *text,
+					   gdouble *x,
+					   gdouble *y)
+{
+	GString * render;
+	render = rsvg_text_render_text_as_string (ctx,tspan,text,x,y);
+	rsvg_render_path (ctx, render->str);
+	g_string_free(render, TRUE);
+}
+
+ArtSVP *
+rsvg_text_render_text_as_svp (DrawingCtx *ctx,
+							  RsvgTspan  *tspan,
+							  const char *text,
+							  gdouble *x,
+							  gdouble *y)
+{
+	GString * render;
+	ArtSVP * output;
+	render = rsvg_text_render_text_as_string (ctx,tspan,text,x,y);
+	output = rsvg_render_path_as_svp (ctx, render->str);
+	g_string_free(render, TRUE);
+	return output;
+}
+
+static gdouble
+rsvg_text_layout_width  (RsvgTextLayout *layout)
+{
+	PangoLayoutIter *iter;
+	gint             offx, offy;
+
+	rsvg_text_layout_get_offsets (layout, &offx, &offy);
+	
+	iter = pango_layout_get_iter (layout->layout);
+	
+	if (iter)
+		{
+			PangoRectangle   rect;
+			PangoLayoutLine *line;
+			
+			line = pango_layout_iter_get_line (iter);
+			
+			pango_layout_iter_get_line_extents (iter, NULL, &rect);
+
+			pango_layout_iter_free (iter);
+			return rect.width / PANGO_SCALE + offx;
+		}
+
+	return 0;
+}
+
+static gdouble
+rsvg_text_width       (DrawingCtx *ctx,
+					   RsvgTspan  *tspan,
+					   const char *text)
+{
+	RsvgTextLayout *layout;
+	RsvgState      *state;
+	gdouble output;
+
+	state = rsvg_state_current(ctx);
+
+	layout = rsvg_text_layout_new (ctx, state, text);
+	layout->span = tspan;
+
+	output = rsvg_text_layout_width (layout);
+	rsvg_text_layout_free (layout);
+
+	return output;
 }
