@@ -27,18 +27,19 @@
 #define RSVG_TEXT_H
 
 #include "rsvg.h"
+#include "rsvg-shapes.h"
 
 G_BEGIN_DECLS
 
 void rsvg_start_text (RsvgHandle *ctx, RsvgPropertyBag *atts);
-void rsvg_start_tspan (RsvgHandle *ctx, RsvgPropertyBag *atts);
-char * make_valid_utf8 (const char *str);
+char * rsvg_make_valid_utf8 (const char *str);
 
-void 
-rsvg_text_render_text (RsvgHandle *ctx,
-					   RsvgState  *state,
-					   const char *text,
-					   const char *id);
+typedef struct _RsvgDefsDrawableText RsvgDefsDrawableText;
+
+struct _RsvgDefsDrawableText {
+	RsvgDefsDrawable super;
+	void * chunk;
+};
 
 G_END_DECLS
 
