@@ -44,6 +44,10 @@
 #include "rsvg-path.h"
 #include "rsvg-paint-server.h"
 
+#ifdef HAVE_SVGZ
+#include <gsf/gsf-utils.h>
+#endif
+
 /*
  * This is configurable at runtime
  */
@@ -52,6 +56,12 @@
 static double internal_dpi_x = RSVG_DEFAULT_DPI_X;
 static double internal_dpi_y = RSVG_DEFAULT_DPI_Y;
 
+static void
+rsvg_drawing_ctx_free (DrawingCtx *handle);
+
+void
+rsvg_drawing_ctx_init(DrawingCtx * handle);
+
 static void
 rsvg_ctx_free_helper (gpointer key, gpointer value, gpointer user_data)
 {
@@ -76,10 +86,8 @@ rsvg_pixmap_destroy (guchar *pixels, gpointer data)
 static void
 rsvg_start_svg (RsvgHandle *ctx, RsvgPropertyBag *atts)
 {
-	int width = -1, height = -1, x = -1, y = -1;
-	int rowstride;
-	art_u8 *pixels;
-	RsvgState *state;
+	int width = -1, height = -1, x = -1, y = -1, i;
+	RsvgState state;
 	gint new_width, new_height;
 	double x_zoom = 1.;
 	double y_zoom = 1.;
@@ -89,6 +97,8 @@ rsvg_start_svg (RsvgHandle *ctx, RsvgPropertyBag *atts)
 	double vbox_x = 0, vbox_y = 0, vbox_w = 0, vbox_h = 0;
 	gboolean has_vbox = FALSE;
 
+	rsvg_state_init(&state);
+
 	if (rsvg_property_bag_size (atts))
 		{
 			/* x & y should be ignored since we should always be the outermost SVG,
@@ -115,8 +125,9 @@ rsvg_start_svg (RsvgHandle *ctx, RsvgPropertyBag *atts)
 					/* apply the sizing function on the *original* width and height
 					   to acquire our real destination size. we'll scale it against
 					   the viewBox's coordinates later */
-					if (ctx->size_func)
+					if (ctx->size_func) {
 						(* ctx->size_func) (&width, &height, ctx->user_data);
+					}
 				}
 			else
 				{
@@ -133,8 +144,9 @@ rsvg_start_svg (RsvgHandle *ctx, RsvgPropertyBag *atts)
 
 					/* apply the sizing function to acquire our new width and height.
 					   we'll scale this against the old values later */
-					if (ctx->size_func)
+					if (ctx->size_func) {
 						(* ctx->size_func) (&new_width, &new_height, ctx->user_data);
+					}
 				}
 
 			/* set these here because % are relative to viewbox */
@@ -157,18 +169,17 @@ rsvg_start_svg (RsvgHandle *ctx, RsvgPropertyBag *atts)
 				}
 
 			/* Scale size of target pixbuf */
-			state = rsvg_state_current (ctx);
 
-			art_affine_identity (state->affine);
+			art_affine_identity (state.affine);
 
 			if (has_vbox && (vbox_x != 0. || vbox_y != 0.))
 				{
 					art_affine_translate (affine, - vbox_x, - vbox_y);
-					art_affine_multiply (state->affine, state->affine, affine);
+					art_affine_multiply (state.affine, state.affine, affine);
 				}
 
 			art_affine_scale (affine, x_zoom, y_zoom);
-			art_affine_multiply (state->affine, state->affine, affine);
+			art_affine_multiply (state.affine, state.affine, affine);
 
 			if (new_width <= 0 || new_height <= 0)
 				{
@@ -183,107 +194,34 @@ rsvg_start_svg (RsvgHandle *ctx, RsvgPropertyBag *atts)
 					g_warning (_("rsvg_start_svg: width too large"));
 					return;
 				}
-			rowstride = (new_width * 4 + 3) & ~3;
-			if (rowstride > INT_MAX / new_height)
-				{
-					/* FIXME: GError here? */
-					g_warning (_("rsvg_start_svg: width too large"));
-					return;
-				}
 
 			/* FIXME: Add GError here if size is too big. */
-
-			pixels = g_try_malloc (rowstride * new_height);
-			if (pixels == NULL)
-				{
-					/* FIXME: GError here? */
-					g_warning (_("rsvg_start_svg: dimensions too large"));
-					return;
-				}
-			memset (pixels, 0, rowstride * new_height);
-			ctx->pixbuf = gdk_pixbuf_new_from_data (pixels,
-													GDK_COLORSPACE_RGB,
-													TRUE, 8,
-													new_width, new_height,
-													rowstride,
-													rsvg_pixmap_destroy,
-													NULL);
 		}
+
+	ctx->new_width = new_width;
+	ctx->new_height = new_height;
+
+	for (i = 0; i < 6; i++)
+		state.personal_affine[i] = state.affine[i];
 	ctx->nest_level = 1;
 	ctx->current_defs_group = NULL;
-}
-
-static void
-rsvg_start_sub_svg (RsvgHandle *ctx, RsvgPropertyBag *atts)
-{
-	int width = -1, height = -1, x = -1, y = -1;
-	double affine[6];
-	const char * id, *value;
-
-	double vbox_x = 0, vbox_y = 0, vbox_w = 0, vbox_h = 0;
-	gboolean has_vbox = FALSE;
-	id = NULL;
-
-	if (rsvg_property_bag_size (atts))
-		{
-			/* x & y should be ignored since we should always be the outermost SVG,
-			   at least for now, but i'll include them here anyway */
-			if ((value = rsvg_property_bag_lookup (atts, "viewBox")))
-				{
-					has_vbox = rsvg_css_parse_vbox (value, &vbox_x, &vbox_y,
-													&vbox_w, &vbox_h);
-				}
-			if ((value = rsvg_property_bag_lookup (atts, "width")))
-				width = rsvg_css_parse_normalized_length (value, ctx->dpi_x, ctx->width, 1);
-			if ((value = rsvg_property_bag_lookup (atts, "height")))
-				height = rsvg_css_parse_normalized_length (value, ctx->dpi_y, ctx->height, 1);
-			if ((value = rsvg_property_bag_lookup (atts, "x")))
-				x = rsvg_css_parse_normalized_length (value, ctx->dpi_x, ctx->width, 1);
-			if ((value = rsvg_property_bag_lookup (atts, "y")))
-				y = rsvg_css_parse_normalized_length (value, ctx->dpi_y, ctx->height, 1);
-			if ((value = rsvg_property_bag_lookup (atts, "id")))
-				id = value;
-
-		}
-	if (has_vbox)
-		{
-			affine[0] = width / vbox_w;
-			affine[1] = 0;
-			affine[2] = 0;
-			affine[3] = height / vbox_h;
-			affine[4] = x - vbox_x * width / vbox_w;
-			affine[5] = y - vbox_y * height / vbox_h;
-			art_affine_multiply(rsvg_state_current(ctx)->affine, affine, 
-								rsvg_state_current(ctx)->affine);	
-		}
-	else
-		{
-			affine[0] = 1;
-			affine[1] = 0;
-			affine[2] = 0;
-			affine[3] = 1;
-			affine[4] = x;
-			affine[5] = y;
-			art_affine_multiply(rsvg_state_current(ctx)->affine, affine, 
-								rsvg_state_current(ctx)->affine);				
-		}
-	rsvg_push_def_group (ctx, id);
-	ctx->nest_level++;
+	ctx->treebase = rsvg_push_def_group(ctx, NULL, state);
 }
 
 static void
 rsvg_end_sub_svg(RsvgHandle *ctx)
 {
-	if (ctx->nest_level-- > 1)
-		rsvg_pop_def_group (ctx);
+	ctx->nest_level--;
+	rsvg_pop_def_group (ctx);
 }
 
 static void
 rsvg_start_g (RsvgHandle *ctx, RsvgPropertyBag *atts)
 {
-	RsvgState *state = rsvg_state_current (ctx);
+	RsvgState state;
 	const char * klazz = NULL, * id = NULL, *value;
-	
+	rsvg_state_init(&state);
+
 	if (rsvg_property_bag_size (atts))
 		{
 			if ((value = rsvg_property_bag_lookup (atts, "class")))
@@ -291,20 +229,38 @@ rsvg_start_g (RsvgHandle *ctx, RsvgPropertyBag *atts)
 			if ((value = rsvg_property_bag_lookup (atts, "id")))
 				id = value;
 
-			rsvg_parse_style_attrs (ctx, state, "g", klazz, id, atts);
+			rsvg_parse_style_attrs (ctx, &state, "g", klazz, id, atts);
 		}	
   
-	rsvg_push_def_group (ctx, id);
-	if (!ctx->in_defs)	
-		rsvg_push_discrete_layer (ctx);
+	rsvg_push_def_group (ctx, id, state);
 }
 
 static void
 rsvg_end_g (RsvgHandle *ctx)
 {
 	rsvg_pop_def_group (ctx);
-	if (!ctx->in_defs)
-		rsvg_pop_discrete_layer (ctx);
+}
+
+static void
+rsvg_start_defs (RsvgHandle *ctx, RsvgPropertyBag *atts)
+{
+	RsvgState state;
+	const char * klazz = NULL, * id = NULL, *value;
+	rsvg_state_init(&state);	
+
+	if (rsvg_property_bag_size (atts))
+		{
+			if ((value = rsvg_property_bag_lookup (atts, "class")))
+				klazz = value;
+			if ((value = rsvg_property_bag_lookup (atts, "id")))
+				id = value;
+
+			rsvg_parse_style_attrs (ctx, &state, "defs", klazz, id, atts);
+		}	
+  
+	/*I don't know if I am proud or discusted by this hack. It seems to 
+	  have the same effect as the spec but not be in its spirit.*/
+	rsvg_push_part_def_group (ctx, id, state);
 }
 
 typedef struct _RsvgSaxHandlerDefs {
@@ -330,6 +286,7 @@ typedef struct _RsvgSaxHandlerGstops {
 /* hide this fact from the general public */
 typedef RsvgSaxHandlerDefs RsvgSaxHandlerTitle;
 typedef RsvgSaxHandlerDefs RsvgSaxHandlerDesc;
+typedef RsvgSaxHandlerDefs RsvgSaxHandlerMetadata;
 
 static void
 rsvg_gradient_stop_handler_free (RsvgSaxHandler *self)
@@ -353,7 +310,7 @@ rsvg_gradient_stop_handler_start (RsvgSaxHandler *self, const xmlChar *name,
 	if (strcmp ((char *)name, "stop"))
 		return;
 	
-	rsvg_state_clone(&state ,rsvg_state_current(z->ctx));
+	rsvg_state_init(&state);
 	
 	if (rsvg_property_bag_size (atts))
 		{
@@ -467,7 +424,7 @@ rsvg_linear_gradient_free (RsvgDefVal *self)
 static void
 rsvg_start_linear_gradient (RsvgHandle *ctx, RsvgPropertyBag *atts)
 {
-	RsvgState *state = rsvg_state_current (ctx);
+	RsvgState state;
 	RsvgLinearGradient *grad = NULL;
 	const char *id = NULL, *value;
 	double x1 = 0., y1 = 0., x2 = 0., y2 = 0.;
@@ -480,6 +437,8 @@ rsvg_start_linear_gradient (RsvgHandle *ctx, RsvgPropertyBag *atts)
 	gboolean got_color = FALSE;
 	int i;
 
+	rsvg_state_init(&state);
+
 	got_x1 = got_x2 = got_y1 = got_y2 = got_spread = got_transform = got_bbox = cloned = shallow_cloned = FALSE;
 		
 	if (rsvg_property_bag_size (atts))
@@ -487,19 +446,19 @@ rsvg_start_linear_gradient (RsvgHandle *ctx, RsvgPropertyBag *atts)
 			if ((value = rsvg_property_bag_lookup (atts, "id")))
 				id = value;
 			if ((value = rsvg_property_bag_lookup (atts, "x1"))) {
-				x1 = rsvg_css_parse_normalized_length (value, ctx->dpi_x, 1, state->font_size);
+				x1 = rsvg_css_parse_normalized_length (value, ctx->dpi_x, 1, state.font_size);
 				got_x1 = TRUE;
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "y1"))) {
-				y1 = rsvg_css_parse_normalized_length (value, ctx->dpi_y, 1, state->font_size);
+				y1 = rsvg_css_parse_normalized_length (value, ctx->dpi_y, 1, state.font_size);
 				got_y1 = TRUE;
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "x2"))) {
-				x2 = rsvg_css_parse_normalized_length (value, ctx->dpi_x, 1, state->font_size);
+				x2 = rsvg_css_parse_normalized_length (value, ctx->dpi_x, 1, state.font_size);
 				got_x2 = TRUE;
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "y2"))) {
-				y2 = rsvg_css_parse_normalized_length (value, ctx->dpi_y, 1, state->font_size);
+				y2 = rsvg_css_parse_normalized_length (value, ctx->dpi_y, 1, state.font_size);
 				got_y2 = TRUE;
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "spreadMethod")))
@@ -532,7 +491,7 @@ rsvg_start_linear_gradient (RsvgHandle *ctx, RsvgPropertyBag *atts)
 				got_bbox = TRUE;
 
 			}
-			rsvg_parse_style_pairs (ctx, state, atts);
+			rsvg_parse_style_pairs (ctx, &state, atts);
 		}
 	
 	/* set up 100% as the default if not gotten */
@@ -540,12 +499,12 @@ rsvg_start_linear_gradient (RsvgHandle *ctx, RsvgPropertyBag *atts)
 		if (obj_bbox)
 			x2 = 1.0;
 		else
-			x2 = rsvg_css_parse_normalized_length ("100%", ctx->dpi_x, (gdouble)ctx->width, state->font_size);
+			x2 = rsvg_css_parse_normalized_length ("100%", ctx->dpi_x, (gdouble)ctx->width, state.font_size);
 	}
 
 	if (xlink_href != NULL)
 		{
-			RsvgLinearGradient * parent = (RsvgLinearGradient*)rsvg_defs_lookup (ctx->defs, xlink_href+1);
+			RsvgLinearGradient * parent = (RsvgLinearGradient*)rsvg_defs_lookup (ctx->defs, xlink_href);
 			if (parent != NULL)
 				{
 					cloned = TRUE;
@@ -613,7 +572,7 @@ rsvg_radial_gradient_free (RsvgDefVal *self)
 static void
 rsvg_start_radial_gradient (RsvgHandle *ctx, RsvgPropertyBag *atts, const char * tag) /* tag for conicalGradient */
 {
-	RsvgState *state = rsvg_state_current (ctx);
+	RsvgState state;
 	RsvgRadialGradient *grad = NULL;
 	const char *id = NULL;
 	double cx = 0., cy = 0., r = 0., fx = 0., fy = 0.;  
@@ -626,6 +585,8 @@ rsvg_start_radial_gradient (RsvgHandle *ctx, RsvgPropertyBag *atts, const char *
 	double affine[6];
 	int i;
 
+	rsvg_state_init(&state);
+
 	got_cx = got_cy = got_r = got_fx = got_fy = got_spread = got_transform = got_bbox = cloned = shallow_cloned = FALSE;
 	
 	if (rsvg_property_bag_size (atts))
@@ -633,24 +594,24 @@ rsvg_start_radial_gradient (RsvgHandle *ctx, RsvgPropertyBag *atts, const char *
 			if ((value = rsvg_property_bag_lookup (atts, "id")))
 				id = value;
 			if ((value = rsvg_property_bag_lookup (atts, "cx"))) {
-				cx = rsvg_css_parse_normalized_length (value, ctx->dpi_x, 1, state->font_size);
+				cx = rsvg_css_parse_normalized_length (value, ctx->dpi_x, 1, state.font_size);
 				got_cx = TRUE;
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "cy"))) {
-				cy = rsvg_css_parse_normalized_length (value, ctx->dpi_y, 1, state->font_size);
+				cy = rsvg_css_parse_normalized_length (value, ctx->dpi_y, 1, state.font_size);
 				got_cy = TRUE;
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "r"))) {
 				r = rsvg_css_parse_normalized_length (value, rsvg_dpi_percentage (ctx), 1, 
-													  state->font_size);
+													  state.font_size);
 				got_r = TRUE;
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "fx"))) {
-				fx = rsvg_css_parse_normalized_length (value, ctx->dpi_x, 1, state->font_size);
+				fx = rsvg_css_parse_normalized_length (value, ctx->dpi_x, 1, state.font_size);
 				got_fx = TRUE;
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "fy"))) {
-				fy = rsvg_css_parse_normalized_length (value, ctx->dpi_y, 1, state->font_size);
+				fy = rsvg_css_parse_normalized_length (value, ctx->dpi_y, 1, state.font_size);
 				got_fy = TRUE;
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "xlink:href")))
@@ -683,12 +644,12 @@ rsvg_start_radial_gradient (RsvgHandle *ctx, RsvgPropertyBag *atts, const char *
 					obj_bbox = FALSE;
 				got_bbox = TRUE;
 			}
-			rsvg_parse_style_pairs (ctx, state, atts);
+			rsvg_parse_style_pairs (ctx, &state, atts);
 		}
 	
 	if (xlink_href != NULL)
 		{
-			RsvgRadialGradient * parent = (RsvgRadialGradient*)rsvg_defs_lookup (ctx->defs, xlink_href+1);
+			RsvgRadialGradient * parent = (RsvgRadialGradient*)rsvg_defs_lookup (ctx->defs, xlink_href);
 			if (parent != NULL)
 				{
 					cloned = TRUE;
@@ -709,19 +670,19 @@ rsvg_start_radial_gradient (RsvgHandle *ctx, RsvgPropertyBag *atts, const char *
 		if (obj_bbox)
 			cx = 0.5;
 		else
-			cx = rsvg_css_parse_normalized_length ("50%", ctx->dpi_x, (gdouble)ctx->width, state->font_size);
+			cx = rsvg_css_parse_normalized_length ("50%", ctx->dpi_x, (gdouble)ctx->width, state.font_size);
 	}
 	if (!got_cy) {
 		if (obj_bbox)
 			cy = 0.5;
 		else
-			cy = rsvg_css_parse_normalized_length ("50%", ctx->dpi_y, (gdouble)ctx->height, state->font_size);
+			cy = rsvg_css_parse_normalized_length ("50%", ctx->dpi_y, (gdouble)ctx->height, state.font_size);
 	}
 	if (!got_r) {
 		if (obj_bbox)
 			r = 0.5;
 		else
-			r  = rsvg_css_parse_normalized_length ("50%", rsvg_dpi_percentage (ctx), rsvg_viewport_percentage((gdouble)ctx->width, (gdouble)ctx->height), state->font_size);
+			r  = rsvg_css_parse_normalized_length ("50%", rsvg_dpi_percentage (ctx), rsvg_viewport_percentage((gdouble)ctx->width, (gdouble)ctx->height), state.font_size);
 	}
 	if (!got_fx) {
 		fx = cx;
@@ -780,7 +741,7 @@ rsvg_pattern_free (RsvgDefVal *self)
 static void
 rsvg_start_pattern (RsvgHandle *ctx, RsvgPropertyBag *atts)
 {
-	RsvgState *state = rsvg_state_current (ctx);
+	RsvgState state;
 	RsvgPattern *pattern = NULL;
 	const char *id = NULL, *value;
 	double x = 0., y = 0., width = 0., height = 0.;
@@ -788,11 +749,14 @@ rsvg_start_pattern (RsvgHandle *ctx, RsvgPropertyBag *atts)
 	const char * xlink_href = NULL;
 	gboolean obj_bbox = TRUE;
 	gboolean obj_cbbox = FALSE;
-	gboolean got_x, got_y, got_width, got_height, got_transform, got_bbox, got_cbbox, cloned, got_vbox;
+	gboolean got_x, got_y, got_width, got_height, got_transform, got_bbox, got_cbbox, cloned, got_vbox, got_aspect_ratio;
 	double affine[6];
 	int i;
+	guint aspect_ratio = RSVG_ASPECT_RATIO_XMID_YMID;
 
-	got_x = got_y = got_width = got_height = got_transform = got_bbox = got_cbbox = cloned = got_vbox = FALSE;
+	rsvg_state_init(&state);
+
+	got_x = got_y = got_width = got_height = got_transform = got_bbox = got_cbbox = cloned = got_vbox = got_aspect_ratio = FALSE;
 		
 	if (rsvg_property_bag_size (atts))
 		{
@@ -804,19 +768,19 @@ rsvg_start_pattern (RsvgHandle *ctx, RsvgPropertyBag *atts)
 													&vbw, &vbh);
 				}
 			if ((value = rsvg_property_bag_lookup (atts, "x"))) {
-				x = rsvg_css_parse_normalized_length (value, ctx->dpi_x, 1, state->font_size);
+				x = rsvg_css_parse_normalized_length (value, ctx->dpi_x, 1, state.font_size);
 				got_x = TRUE;
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "y"))) {
-				y = rsvg_css_parse_normalized_length (value, ctx->dpi_y, 1, state->font_size);
+				y = rsvg_css_parse_normalized_length (value, ctx->dpi_y, 1, state.font_size);
 				got_y = TRUE;
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "width"))) {
-				width = rsvg_css_parse_normalized_length (value, ctx->dpi_x, 1, state->font_size);
+				width = rsvg_css_parse_normalized_length (value, ctx->dpi_x, 1, state.font_size);
 				got_width = TRUE;
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "height"))) {
-				height = rsvg_css_parse_normalized_length (value, ctx->dpi_y, 1, state->font_size);
+				height = rsvg_css_parse_normalized_length (value, ctx->dpi_y, 1, state.font_size);
 				got_height = TRUE;
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "xlink:href")))
@@ -837,11 +801,15 @@ rsvg_start_pattern (RsvgHandle *ctx, RsvgPropertyBag *atts)
 					obj_cbbox = TRUE;					
 				got_cbbox = TRUE;
 			}
+			if ((value = rsvg_property_bag_lookup (atts, "preserveAspectRatio")))
+				aspect_ratio = rsvg_css_parse_aspect_ratio (value);
+
+
 		}
 
 	if (xlink_href != NULL)
 		{
-			RsvgPattern * parent = (RsvgPattern*)rsvg_defs_lookup (ctx->defs, xlink_href+1);
+			RsvgPattern * parent = (RsvgPattern*)rsvg_defs_lookup (ctx->defs, xlink_href);
 			if (parent != NULL)
 				{
 					cloned = TRUE;
@@ -865,6 +833,11 @@ rsvg_start_pattern (RsvgHandle *ctx, RsvgPropertyBag *atts)
 	else
 		art_affine_identity(pattern->affine);
 
+	if (got_aspect_ratio)
+		pattern->preserve_aspect_ratio = aspect_ratio;
+	else
+		pattern->preserve_aspect_ratio = RSVG_ASPECT_RATIO_XMID_YMID;
+
 	/* gradient inherits parent/cloned information unless it's explicity gotten */
 	pattern->obj_bbox = (cloned && !got_bbox) ? pattern->obj_bbox : obj_bbox;
 	pattern->obj_cbbox = (cloned && !got_cbbox) ? pattern->obj_cbbox : obj_cbbox;
@@ -878,9 +851,7 @@ rsvg_start_pattern (RsvgHandle *ctx, RsvgPropertyBag *atts)
 	pattern->vbh = (cloned && !got_vbox) ? pattern->vbh : vbh;
 	pattern->vbox = (cloned && !got_vbox) ? pattern->vbox : got_vbox;
 
-	ctx->in_defs++;		
-
-	pattern->g = &(rsvg_push_def_group (ctx, NULL)->super);
+	pattern->g = &(rsvg_push_part_def_group (ctx, NULL, state)->super);
 }
 
 
@@ -1070,6 +1041,7 @@ rsvg_start_desc (RsvgHandle *ctx, RsvgPropertyBag *atts)
 	handler->super.end_element   = rsvg_desc_handler_end;
 	handler->ctx = ctx;
 
+	ctx->desc = g_string_new (NULL);
 	ctx->handler = &handler->super;
 }
 
@@ -1143,10 +1115,103 @@ rsvg_start_title (RsvgHandle *ctx, RsvgPropertyBag *atts)
 	handler->super.end_element   = rsvg_title_handler_end;
 	handler->ctx = ctx;
 
+	ctx->title = g_string_new (NULL);
 	ctx->handler = &handler->super;
 }
 
 /* end title */
+
+/* start metadata */
+
+static void
+rsvg_metadata_handler_free (RsvgSaxHandler *self)
+{
+	g_free (self);
+}
+
+static void
+rsvg_metadata_handler_characters (RsvgSaxHandler *self, const xmlChar *ch, int len)
+{
+	RsvgSaxHandlerDesc *z = (RsvgSaxHandlerDesc *)self;
+	RsvgHandle *ctx = z->ctx;
+
+	char * string = NULL;
+	char * utf8 = NULL;
+
+	/* This isn't quite the correct behavior - in theory, any graphics
+	   element may contain a metadata or desc element */
+
+	if (!ch || !len)
+		return;
+
+	string = g_strndup (ch, len);
+	if (!g_utf8_validate (string, -1, NULL))
+		{
+			utf8 = rsvg_make_valid_utf8 (string);
+			g_free (string);
+			string = utf8;
+		}
+
+	g_string_append (ctx->metadata, string);
+	g_free (string);
+}
+
+static void
+rsvg_metadata_props_enumerate (const char * key,
+							   const char * value,
+							   gpointer user_data)
+{
+	GString * metadata = (GString *)user_data;
+	g_string_append_printf (metadata, "%s=\"%s\" ", key, value);
+}
+
+static void
+rsvg_metadata_handler_start (RsvgSaxHandler *self, const xmlChar *name,
+							 RsvgPropertyBag *atts)
+{
+	RsvgSaxHandlerMetadata *z = (RsvgSaxHandlerMetadata *)self;
+	RsvgHandle *ctx = z->ctx;
+
+	g_string_append_printf (ctx->metadata, "<%s ", name);
+	rsvg_property_bag_enumerate (atts, rsvg_metadata_props_enumerate, ctx->metadata);
+	g_string_append (ctx->metadata, ">\n");
+}
+
+static void
+rsvg_metadata_handler_end (RsvgSaxHandler *self, const xmlChar *name)
+{
+	RsvgSaxHandlerMetadata *z = (RsvgSaxHandlerMetadata *)self;
+	RsvgHandle *ctx = z->ctx;
+	
+	if (!strcmp((char *)name, "metadata"))
+		{
+			if (ctx->handler != NULL)
+				{
+					ctx->handler->free (ctx->handler);
+					ctx->handler = NULL;
+				}
+		}
+	else
+		g_string_append_printf (ctx->metadata, "</%s>\n", name);
+}
+
+static void
+rsvg_start_metadata (RsvgHandle *ctx, RsvgPropertyBag *atts)
+{
+	RsvgSaxHandlerMetadata *handler = g_new0 (RsvgSaxHandlerMetadata, 1);
+	
+	handler->super.free = rsvg_metadata_handler_free;
+	handler->super.characters = rsvg_metadata_handler_characters;
+	handler->super.start_element = rsvg_metadata_handler_start;
+	handler->super.end_element   = rsvg_metadata_handler_end;
+	handler->ctx = ctx;
+
+	ctx->metadata = g_string_new (NULL);
+	ctx->handler = &handler->super;
+}
+
+/* end metadata */
+
 static void
 rsvg_start_element (void *data, const xmlChar *name,
 					const xmlChar ** atts)
@@ -1154,6 +1219,13 @@ rsvg_start_element (void *data, const xmlChar *name,
 	RsvgHandle *ctx = (RsvgHandle *)data;
 
 	RsvgPropertyBag * bag;
+  
+	RsvgDimentionData * newdimention;
+	newdimention = g_new(RsvgDimentionData, 1);
+	newdimention->width = ctx->width;
+	newdimention->height = ctx->height;
+	newdimention->em = rsvg_state_current_font_size(ctx);
+	ctx->dimentions = g_slist_prepend(ctx->dimentions, newdimention);
 
 	bag = rsvg_property_bag_new(atts);
 
@@ -1165,8 +1237,6 @@ rsvg_start_element (void *data, const xmlChar *name,
 		}
 	else
 		{
-			rsvg_state_push(ctx);
-			
 			if (!strcmp ((char *)name, "svg"))
 				{
 					if (!ctx->nest_level)
@@ -1177,12 +1247,9 @@ rsvg_start_element (void *data, const xmlChar *name,
 			else if (!strcmp ((char *)name, "g"))
 				rsvg_start_g (ctx, bag);
 			else if (!strcmp ((char *)name, "symbol"))
-				{
-					ctx->in_defs++;
-					rsvg_start_g (ctx, bag);
-				}
+				rsvg_start_symbol (ctx, bag);
 			else if (!strcmp ((char *)name, "defs"))
-				ctx->in_defs++;	
+				rsvg_start_defs(ctx, bag);	
 			else if (!strcmp ((char *)name, "path"))
 				rsvg_start_path (ctx, bag);
 			else if (!strcmp ((char *)name, "line"))
@@ -1209,25 +1276,16 @@ rsvg_start_element (void *data, const xmlChar *name,
 				rsvg_start_title (ctx, bag);
 			else if (!strcmp ((char *)name, "desc"))
 				rsvg_start_desc (ctx, bag);
+			else if (!strcmp ((char *)name, "metadata"))
+				rsvg_start_metadata (ctx, bag);
 			else if (!strcmp ((char *)name, "mask"))
-				{
-					ctx->in_defs++;
-					rsvg_start_mask(ctx, bag);
-				}
+				rsvg_start_mask(ctx, bag);
 			else if (!strcmp ((char *)name, "clipPath"))
-				{
-					ctx->in_defs++;
-					rsvg_start_clip_path(ctx, bag);
-				}
+				rsvg_start_clip_path(ctx, bag);
 			else if (!strcmp ((char *)name, "marker"))
-				{
-					ctx->in_defs++;
-					rsvg_start_marker (ctx, bag);
-				}
+				rsvg_start_marker (ctx, bag);
 			else if (!strcmp ((char *)name, "switch"))
-				{
-					ctx->in_switch++;
-				}
+				ctx->in_switch++;
 			/* see conicalGradient discussion above */
 			else if (!strcmp ((char *)name, "linearGradient"))
 				rsvg_start_linear_gradient (ctx, bag);
@@ -1249,6 +1307,13 @@ rsvg_end_element (void *data, const xmlChar *name)
 {
 	RsvgHandle *ctx = (RsvgHandle *)data;
 	
+	GSList * link = g_slist_nth(ctx->dimentions, 0);
+	RsvgDimentionData * dead_dimention = (RsvgDimentionData *)link->data;
+	ctx->width = dead_dimention->width;
+	ctx->height = dead_dimention->height;
+	g_free (dead_dimention);
+	ctx->dimentions = g_slist_delete_link(ctx->dimentions, link);
+
 	if (ctx->handler_nest > 0 && ctx->handler != NULL)
 		{
 			if (ctx->handler->end_element != NULL)
@@ -1268,37 +1333,23 @@ rsvg_end_element (void *data, const xmlChar *name)
 			else if (!strcmp ((char *)name, "svg"))
 				rsvg_end_sub_svg (ctx);
 			else if (!strcmp ((char *)name, "symbol"))
-				{
-					ctx->in_defs--;
-					rsvg_end_g (ctx);
-				}
+				rsvg_end_g (ctx);
 			else if (!strcmp ((char *)name, "filter"))
 				rsvg_end_filter (ctx);
-			else if (!strcmp ((char *)name, "defs")) {
-				ctx->in_defs--;	
-			}
-			else if (!strcmp ((char *)name, "mask")){
+			else if (!strcmp ((char *)name, "defs"))
+				rsvg_end_g (ctx);	
+			else if (!strcmp ((char *)name, "mask"))
 				rsvg_end_mask(ctx);
-				ctx->in_defs--;				
-			}
-			else if (!strcmp ((char *)name, "clipPath")){
+			else if (!strcmp ((char *)name, "clipPath"))
 				rsvg_end_clip_path(ctx);
-				ctx->in_defs--;				
-			}
-			else if (!strcmp ((char *)name, "marker")){
+			else if (!strcmp ((char *)name, "marker"))
 				rsvg_pop_def_group(ctx);
-				ctx->in_defs--;
-			}
 			else if (!strcmp ((char *)name, "switch"))
 				{
 					ctx->in_switch--;
 				}
-			else if (!strcmp ((char *)name, "pattern")) {
-				ctx->in_defs--;	
+			else if (!strcmp ((char *)name, "pattern"))
 				rsvg_pop_def_group(ctx);
-			}
-
-			rsvg_state_pop(ctx);
 		}
 }
 
@@ -1346,27 +1397,65 @@ rsvg_entity_decl (void *data, const xmlChar *name, int type,
 static void
 rsvg_error_cb (void *data, const char *msg, ...)
 {
+#ifdef G_ENABLE_DEBUG
 	va_list args;
 	
 	va_start (args, msg);
 	vfprintf (stderr, msg, args);
 	va_end (args);
+#endif
 }
 
+/* TODO: this is indempotent, but not exactly threadsafe */
 static xmlSAXHandler rsvgSAXHandlerStruct;
+static gboolean rsvgSAXHandlerStructInited = FALSE;
 
 static void rsvg_SAX_handler_struct_init()
 {
-	rsvgSAXHandlerStruct.getEntity = rsvg_get_entity;
-    rsvgSAXHandlerStruct.entityDecl = rsvg_entity_decl;
-    rsvgSAXHandlerStruct.characters = rsvg_characters;
-    rsvgSAXHandlerStruct.error = rsvg_error_cb;
-	rsvgSAXHandlerStruct.error = rsvg_error_cb;
-	rsvgSAXHandlerStruct.cdataBlock = rsvg_characters;
-    rsvgSAXHandlerStruct.startElement = rsvg_start_element;
-    rsvgSAXHandlerStruct.endElement = rsvg_end_element;
+	if(!rsvgSAXHandlerStructInited) 
+		{
+			rsvgSAXHandlerStructInited = TRUE;
+
+			memset(&rsvgSAXHandlerStruct, 0, sizeof(rsvgSAXHandlerStruct));
+			
+			rsvgSAXHandlerStruct.getEntity = rsvg_get_entity;
+			rsvgSAXHandlerStruct.entityDecl = rsvg_entity_decl;
+			rsvgSAXHandlerStruct.characters = rsvg_characters;
+			rsvgSAXHandlerStruct.error = rsvg_error_cb;
+			rsvgSAXHandlerStruct.cdataBlock = rsvg_characters;
+			rsvgSAXHandlerStruct.startElement = rsvg_start_element;
+			rsvgSAXHandlerStruct.endElement = rsvg_end_element;
+		}
 }
 
+gchar *
+rsvg_get_base_uri_from_filename(const gchar * file_name)
+{
+	gchar *base_uri;
+	int i, last;
+
+	last = 0;
+	for (i = 0; file_name[i] != '\0'; i++)
+		if (file_name[i] == G_DIR_SEPARATOR)
+			last = i;
+
+	base_uri = g_new(gchar, i + 2);
+  
+	for (i = 0; i <= last; i++)
+		base_uri[i] = file_name[i];
+
+	base_uri[i] = '\0';
+	return base_uri;
+}
+
+/**
+ * Set the base URI for this SVG
+ *
+ * @handle: A #RsvgHandle
+ * @base_uri: 
+ *
+ * Since: 2.9 (really present in 2.8 as well)
+ */
 void rsvg_handle_set_base_uri (RsvgHandle *handle,
 							   const char *base_uri)
 {
@@ -1374,9 +1463,17 @@ void rsvg_handle_set_base_uri (RsvgHandle *handle,
 		if (handle->base_uri)
 			g_free (handle->base_uri);
 		handle->base_uri = g_strdup (base_uri);
+		rsvg_defs_set_base_uri(handle->defs, handle->base_uri);
 	}
 }
 
+/**
+ * Gets the base uri for this RsvgHandle.
+ * @handle: A #RsvgHandle
+ *
+ * Returns: the base uri, possibly null
+ * Since: 2.9 (really present in 2.8 as well)
+ */
 G_CONST_RETURN char *rsvg_handle_get_base_uri (RsvgHandle *handle)
 {
 	return handle->base_uri;
@@ -1453,27 +1550,41 @@ rsvg_handle_close_impl (RsvgHandle  *handle,
       g_propagate_error (error, real_error);
       return FALSE;
       }*/
+	handle->finished = TRUE;
+
 	return TRUE;
 }
 
 static void
 rsvg_state_free_func(gpointer data, gpointer user_data)
 {
-	RsvgHandle * ctx = (RsvgHandle *)user_data;
+	DrawingCtx * ctx = (DrawingCtx *)user_data;
 	rsvg_state_finalize((RsvgState *)data);
 	g_mem_chunk_free(ctx->state_allocator, data);
 }
 
+static void
+rsvg_drawing_ctx_free (DrawingCtx *handle)
+{
+	rsvg_defs_free (handle->defs);
+	
+	g_slist_foreach(handle->state, rsvg_state_free_func, (gpointer)handle);
+	g_slist_free (handle->state);
+
+	if (handle->base_uri)
+		g_free (handle->base_uri);
+
+	g_mem_chunk_destroy(handle->state_allocator);
+
+	if (handle->pango_context != NULL)
+		g_object_unref (handle->pango_context);
+
+	g_free (handle);
+}
+
 void
 rsvg_handle_free_impl (RsvgHandle *handle)
 {
-	if (handle->pango_context != NULL)
-		g_object_unref (handle->pango_context);
-	rsvg_defs_free (handle->defs);
-	
-	g_slist_foreach(handle->state, rsvg_state_free_func, (gpointer)handle);
-	g_slist_free (handle->state);
-	
 	g_hash_table_foreach (handle->entities, rsvg_ctx_free_helper, NULL);
 	g_hash_table_destroy (handle->entities);
 	
@@ -1481,22 +1592,38 @@ rsvg_handle_free_impl (RsvgHandle *handle)
 	
 	if (handle->user_data_destroy)
 		(* handle->user_data_destroy) (handle->user_data);
-	if (handle->pixbuf)
-		g_object_unref (handle->pixbuf);
 
 	if (handle->title)
 		g_string_free (handle->title, TRUE);
 	if (handle->desc)
-	    	g_string_free (handle->desc, TRUE);
-
+		g_string_free (handle->desc, TRUE);
+	if (handle->metadata)
+		g_string_free (handle->metadata, TRUE);
 	if (handle->base_uri)
 		g_free (handle->base_uri);
 
-	g_mem_chunk_destroy(handle->state_allocator);
-
 	g_free (handle);
 }
 
+/**
+ * rsvg_handle_get_metadata:
+ * @handle: An #RsvgHandle
+ *
+ * Returns the SVG's metadata in UTF-8 or %NULL. You must make a copy
+ * of this metadata if you wish to use it after #handle has been freed.
+ *
+ * Returns: The SVG's title
+ *
+ * Since: 2.9
+ */
+G_CONST_RETURN char *rsvg_handle_get_metadata (RsvgHandle *handle)
+{
+	if (handle->metadata)
+		return handle->metadata->str;
+	else
+		return NULL;
+}
+
 /**
  * rsvg_handle_get_title:
  * @handle: An #RsvgHandle
@@ -1557,13 +1684,63 @@ rsvg_handle_new (void)
 	handle->close = rsvg_handle_close_impl;
 	handle->free  = rsvg_handle_free_impl;
 
+	handle->finished = 0;
+
 	return handle;
 }
 
+static DrawingCtx * 
+rsvg_new_drawing_ctx(RsvgHandle * handle)
+{
+	art_u8 *pixels;
+	int rowstride, new_width, new_height;
+	DrawingCtx * draw;
+	draw = g_new(DrawingCtx, 1);	
+
+	draw->state = NULL;
+	/* should this be G_ALLOC_ONLY? */
+	draw->state_allocator = g_mem_chunk_create (RsvgState, 256, G_ALLOC_AND_FREE);
+
+	draw->bbox.x0 = draw->bbox.y0 = draw->bbox.x1 = draw->bbox.y1 = 0;
+
+	new_width = handle->new_width;
+	new_height = handle->new_height;
+	rowstride = (new_width * 4 + 3) & ~3;
+	if (new_height <= 0 || rowstride > INT_MAX / new_height)
+		{
+			/* FIXME: GError here? */
+			g_warning (_("rsvg_start_svg: width too large"));
+			return NULL;
+		}
+	pixels = g_try_malloc (rowstride * new_height);
+	if (pixels == NULL)
+		{
+			/* FIXME: GError here? */
+			g_warning (_("rsvg_start_svg: dimensions too large"));
+			return NULL;
+		}
+	memset (pixels, 0, rowstride * new_height);
+	draw->pixbuf = gdk_pixbuf_new_from_data (pixels,
+											 GDK_COLORSPACE_RGB,
+											 TRUE, 8,
+											 new_width, new_height,
+											 rowstride,
+											 rsvg_pixmap_destroy,
+											 NULL);
+	draw->defs = handle->defs;
+	draw->base_uri = g_strdup(handle->base_uri);
+	draw->dpi_x = handle->dpi_x;
+	draw->dpi_y = handle->dpi_y;
+	draw->pango_context = NULL;
+
+	rsvg_state_push(draw);
+
+	return draw;
+}
+
 void
 rsvg_handle_init (RsvgHandle * handle)
 {
-	handle->state = NULL;
 	handle->defs = rsvg_defs_new ();
 	handle->handler_nest = 0;
 	handle->entities = g_hash_table_new (g_str_hash, g_str_equal);
@@ -1576,13 +1753,9 @@ rsvg_handle_init (RsvgHandle * handle)
 	
 	handle->ctxt = NULL;
 	handle->current_defs_group = NULL;
-	handle->title = g_string_new (NULL);
-	handle->desc = g_string_new (NULL);
+	handle->treebase = NULL;
 
-	/* should this be G_ALLOC_ONLY? */
-	handle->state_allocator = g_mem_chunk_create (RsvgState, 256, G_ALLOC_AND_FREE);
-
-	rsvg_state_push(handle);
+	handle->dimentions = NULL;
 }
 
 /**
@@ -1766,12 +1939,22 @@ rsvg_handle_close (RsvgHandle  *handle,
 GdkPixbuf *
 rsvg_handle_get_pixbuf (RsvgHandle *handle)
 {
+	GdkPixbuf * output;
+	DrawingCtx * draw;
 	g_return_val_if_fail (handle != NULL, NULL);
-	
-	if (handle->pixbuf)
-		return g_object_ref (handle->pixbuf);
 
-	return NULL;
+	if (!handle->finished)
+		return NULL;
+
+	draw = rsvg_new_drawing_ctx(handle);
+	if (!draw)
+		return NULL;
+
+	rsvg_defs_drawable_draw((RsvgDefsDrawable *)handle->treebase, draw, 0);
+	output = draw->pixbuf;
+	rsvg_drawing_ctx_free(draw);
+	
+	return output;
 }
 
 /**
@@ -1787,3 +1970,44 @@ rsvg_handle_free (RsvgHandle *handle)
 		(*handle->free) (handle);
 }
 
+#ifdef HAVE_GNOME_VFS
+#include <libgnomevfs/gnome-vfs.h>
+#endif
+
+/**
+ * rsvg_init:
+ *
+ * Initializes librsvg
+ * Since: 2.9
+ **/
+void 
+rsvg_init (void)
+{
+	g_type_init ();
+
+#ifdef HAVE_SVGZ
+	gsf_init ();
+#endif
+
+	xmlInitParser ();
+
+#ifdef HAVE_GNOME_VFS
+	gnome_vfs_init();
+#endif
+}
+
+/**
+ * rsvg_term:
+ *
+ * De-initializes librsvg
+ * Since: 2.9
+ **/
+void
+rsvg_term (void)
+{
+#ifdef HAVE_SVGZ
+	gsf_shutdown ();
+#endif
+
+	xmlCleanupParser ();
+}
