@@ -59,6 +59,9 @@ void        rsvg_set_default_dpi          (double dpi);
 void        rsvg_handle_set_dpi           (RsvgHandle * handle, double dpi);
 #endif
 
+void        rsvg_init (void);
+void        rsvg_term (void);
+
 void        rsvg_set_default_dpi_x_y          (double dpi_x, double dpi_y);
 void        rsvg_handle_set_dpi_x_y           (RsvgHandle * handle, double dpi_x, double dpi_y);
 
@@ -76,6 +79,12 @@ gboolean    rsvg_handle_close             (RsvgHandle      *handle,
 GdkPixbuf  *rsvg_handle_get_pixbuf        (RsvgHandle      *handle);
 void        rsvg_handle_free              (RsvgHandle      *handle);
 
+G_CONST_RETURN char *
+rsvg_handle_get_base_uri (RsvgHandle *handle);
+void rsvg_handle_set_base_uri (RsvgHandle *handle,
+							   const char *base_uri);
+
+
 /* Convenience API */
 
 GdkPixbuf  *rsvg_pixbuf_from_file                  (const gchar  *file_name,
@@ -103,6 +112,7 @@ GdkPixbuf  *rsvg_pixbuf_from_file_at_zoom_with_max (const gchar  *file_name,
 
 G_CONST_RETURN char *rsvg_handle_get_title         (RsvgHandle *handle);
 G_CONST_RETURN char *rsvg_handle_get_desc          (RsvgHandle *handle);
+G_CONST_RETURN char *rsvg_handle_get_metadata      (RsvgHandle *handle);
 
 /* Extended Convenience API */
 
